@@ -2,7 +2,7 @@
                           cmdlineoptions.cpp  -  description
                              -------------------
     begin                : Sun Nov 25 2001
-    copyright            : (C) 2001 by Andr Simon
+    copyright            : (C) 2001 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -24,16 +24,17 @@ using namespace std;
 */
 CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
     numberSpaces(0),
+    lineNrWidth(5),
+    lineNrStart(0),
     wrappingStyle(highlight::WRAP_DISABLED),
     outputType (highlight::HTML),
-    cssOutFilename("highlight.css"),
     opt_language (false),
-    opt_include_css (false),
+    opt_include_style (false),
     opt_help (false),
     opt_version (false),
     opt_verbose (false),
     opt_linenumbers (false),
-    opt_css_style (false),
+    opt_style (false),
     opt_batch_mode (false),
     opt_fragment (false) ,
     opt_attach_line_anchors (false),
@@ -43,10 +44,15 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
     opt_quiet(false),
     opt_xslfo_fop(false),
     opt_replacequotes(false),
+    opt_babel(false),
     opt_print_progress(false),
     opt_fill_zeroes(false),
+    opt_stylepath_explicit(false),
+    opt_force_output(false),
+    opt_ordered_list(false),
     configFileRead(false),
-    helpLang("en")
+    helpLang("en"),
+    encodingName("ISO-8859-1")
 {
 
   loadConfigurationFile();
@@ -58,13 +64,13 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         {OPT_IN, 1, 0, S_OPT_IN},
         {OPT_SYNTAX, 1, 0, S_OPT_SYNTAX},
         {OPT_VERBOSE, 0, 0, S_OPT_VERBOSE},
-        {OPT_INCLUDECSS, 0, 0, S_OPT_INCLUDECSS},
+        {OPT_INC_STYLE, 0, 0, S_OPT_INC_STYLE},
         {OPT_HELP, 0, 0, S_OPT_HELP},
         {OPT_HELPINT, 1, 0, S_OPT_HELPINT},
         {OPT_LINENO,0,0,S_OPT_LINENO},
         {OPT_STYLE, 1,0,S_OPT_STYLE},
-        {OPT_CSSOUT, 1, 0,S_OPT_CSSOUT},
-        {OPT_CSSIN, 1, 0,S_OPT_CSSIN},
+        {OPT_STYLE_OUT, 1, 0,S_OPT_STYLE_OUT},
+        {OPT_STYLE_IN, 1, 0,S_OPT_STYLE_IN},
         {OPT_DELTABS,1,0,S_OPT_DELTABS},
         {OPT_XHTML, 0,0,S_OPT_XHTML},
         {OPT_RTF, 0,0,S_OPT_RTF},
@@ -87,11 +93,28 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         {OPT_WRAP,0,0,S_OPT_WRAP},
         {OPT_WRAPSIMPLE,0,0,S_OPT_WRAPSIMPLE},
         {OPT_QUIET,0,0,S_OPT_QUIET},
-        //remove as soon as APAche fixes the bug in FOP (0.20.5)
-        {OPT_FOP,0,0,S_OPT_FOP},
         {OPT_REPLACE_QUOTES,0,0,S_OPT_REPLACE_QUOTES},
         {OPT_PROGRESSBAR,0,0,S_OPT_PROGRESSBAR},
         {OPT_FILLZEROES,0,0,S_OPT_FILLZEROES},
+        {OPT_ENCODING,1,0,S_OPT_ENCODING},
+        // -- Kafka begins
+        {OPT_BASE_FONT,1,0,S_OPT_BASE_FONT},
+        {OPT_BASE_FONT_SIZE,1,0,S_OPT_BASE_FONT_SIZE},
+        // -- Kafka ends
+        {OPT_ORDERED_LIST,0,0,S_OPT_ORDERED_LIST},
+        {OPT_LNR_WIDTH,1,0,S_OPT_LNR_WIDTH},
+        {OPT_LNR_START,1,0,S_OPT_LNR_START},
+        {OPT_BABEL,0,0,S_OPT_BABEL},
+
+        //remove as soon as APAche fixes the bug in FOP (0.20.5)
+        {OPT_FOP,0,0,S_OPT_FOP},
+
+        //deprecated
+        {OPT_CSSOUT,1,0,0},
+        {OPT_CSSIN,1,0,0},
+        {OPT_INC_CSS,0,0,0},
+        {OPT_FORCE_OUTPUT,0,0,0},
+
         {0, 0, 0, 0}
       };
 
@@ -103,11 +126,25 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
 
       switch (c)
         {
-        case 0:   //nur long options
-          if (long_options[option_index].name==OPT_VERSION)
-            {
+        case 0:   // long options
+          if (long_options[option_index].name==OPT_VERSION) {
               opt_version = true;
             }
+            if (long_options[option_index].name==OPT_CSSOUT) {
+                styleOutFilename=string(optarg);
+                printDeprecatedWarning(OPT_CSSOUT, OPT_STYLE_OUT);
+            }
+            else if (long_options[option_index].name==OPT_CSSIN) {
+                styleInFilename=string(optarg);
+                printDeprecatedWarning(OPT_CSSIN, OPT_STYLE_IN);
+            }
+            else if (long_options[option_index].name==OPT_INC_CSS) {
+                opt_include_style = true;
+                printDeprecatedWarning(OPT_INC_CSS, OPT_INC_STYLE);
+            }
+            else if (long_options[option_index].name==OPT_FORCE_OUTPUT) {
+                opt_force_output = true;
+            }
           break;
         case S_OPT_OUT:
           outFilename=string(optarg);
@@ -115,11 +152,12 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         case S_OPT_IN:
           inputFileNames.push_back(string(optarg));
           break;
-        case S_OPT_CSSOUT:
-          cssOutFilename=string(optarg);
+        case S_OPT_STYLE_OUT:
+          styleOutFilename=string(optarg);
+          opt_stylepath_explicit=true;
           break;
-        case S_OPT_CSSIN:
-          cssInFilename=string(optarg);
+        case S_OPT_STYLE_IN:
+          styleInFilename=string(optarg);
           break;
         case S_OPT_VERBOSE:
           opt_verbose = true;
@@ -127,8 +165,8 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         case S_OPT_QUIET:
           opt_quiet = true;
           break;
-        case S_OPT_INCLUDECSS:
-          opt_include_css = true;
+         case S_OPT_INC_STYLE:
+          opt_include_style = true;
           break;
         case S_OPT_HELPINT:
           helpLang=string(optarg);
@@ -139,18 +177,18 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
           opt_linenumbers = true;
           break;
         case '?':
-          opt_help = true;
+          //opt_help = true;
           break;
         case S_OPT_STYLE:
           styleName=string(optarg);
-          opt_css_style = true;
+          opt_style = true;
           break;
         case S_OPT_SYNTAX:
           language=string(optarg);
           opt_language = true;
           break;
         case S_OPT_DELTABS:
-          numberSpaces = StringTools::str2int (string(optarg));
+          numberSpaces = StringTools::str2int (string(optarg), true);
           break;
         case S_OPT_XHTML:
           outputType=highlight::XHTML;
@@ -195,6 +233,9 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         case S_OPT_FORMATSTYLE:
           indentScheme =string(optarg);
           break;
+        case S_OPT_ENCODING:
+          encodingName =string(optarg);
+          break;
         case S_OPT_DATADIR:
           dataDir=validateDirPath(string(optarg));
           break;
@@ -216,12 +257,32 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
          case S_OPT_REPLACE_QUOTES:
           opt_replacequotes=true;
           break;
+         case S_OPT_BABEL:
+          opt_babel=true;
+          break;
          case S_OPT_PROGRESSBAR:
           opt_print_progress=true;
           break;
          case S_OPT_FILLZEROES:
           opt_fill_zeroes=true;
           break;
+          // -- Kafka begins
+         case S_OPT_BASE_FONT:
+          baseFont=string(optarg);
+          break;
+         case S_OPT_BASE_FONT_SIZE:
+          baseFontSize=string(optarg);
+          break;
+         // -- Kafka ends
+         case S_OPT_LNR_WIDTH:
+          lineNrWidth=StringTools::str2int(string(optarg), true);
+          break;
+         case S_OPT_LNR_START:
+          lineNrStart=StringTools::str2int(string(optarg), true);
+          break;
+         case S_OPT_ORDERED_LIST:
+            opt_ordered_list = opt_linenumbers = true;
+          break;
 
         default:
           cerr <<"higlight: Unknown option " <<c<< endl;
@@ -230,12 +291,12 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
 
   if (optind < argc)   //still args left
     {
-      if  (inputFileNames.empty()){
+      if  (inputFileNames.empty()) {
         while (optind < argc){
           inputFileNames.push_back(string(argv[optind++]));
          }
       }
-    } else if (inputFileNames.empty()){
+    } else if (inputFileNames.empty()) {
        inputFileNames.push_back("");
     }
   if (printDebugInfo() && configFileRead) {
@@ -248,15 +309,12 @@ CmdLineOptions::~CmdLineOptions(){
 
 const string &CmdLineOptions::getSingleOutFilename()
   {
-   if (!inputFileNames.empty() && !outDirectory.empty()){
-
+   if (!inputFileNames.empty() && !outDirectory.empty()) {
       if (outFilename.empty()) {
         outFilename = outDirectory;
         int delim = getSingleInFilename().find_last_of(Platform::pathSeparator)+1;
         outFilename += getSingleInFilename().substr((delim>-1)?delim:0)
                        + getOutFileSuffix();
-      } else {
-         outFilename = outDirectory+outFilename;
       }
    }
    return outFilename;
@@ -275,13 +333,15 @@ const string &CmdLineOptions::getOutDirectory()
     return outDirectory;
   }
 
-const string &CmdLineOptions::getCssOutFilename() const
+const string CmdLineOptions::getStyleOutFilename() const
   {
-    return cssOutFilename;
+      if (!styleOutFilename.empty()) return styleOutFilename;
+      return (outputType==highlight::HTML ||
+              outputType==highlight::XHTML)? "highlight.css":"highlight.sty";
   }
-const string &CmdLineOptions::getCssInFilename() const
+const string &CmdLineOptions::getStyleInFilename() const
   {
-    return cssInFilename;
+    return styleInFilename;
   }
 int CmdLineOptions::getNumberSpaces() const
   {
@@ -303,11 +363,18 @@ bool CmdLineOptions::quietMode()const
   {
     return opt_quiet;
   }
-bool CmdLineOptions::includeCssDefinition()const
+bool CmdLineOptions::includeStyleDef()const
   {
-    return opt_include_css && (outputType==highlight::HTML ||
-                                outputType==highlight::XHTML);
+      return opt_include_style;
   }
+
+bool CmdLineOptions::formatSupportsExtStyle(){
+      return outputType==highlight::HTML ||
+             outputType==highlight::XHTML ||
+             outputType==highlight::LATEX ||
+             outputType==highlight::TEX;
+}
+
 bool CmdLineOptions::printLineNumbers()const
   {
     return opt_linenumbers;
@@ -315,7 +382,7 @@ bool CmdLineOptions::printLineNumbers()const
 
 string CmdLineOptions::getStyleName()const
   {
-    return ( ( opt_css_style) ? styleName+".style" : "kwrite.style" );
+    return ( ( opt_style) ? styleName+".style" : "kwrite.style" );
   }
 bool CmdLineOptions::enableBatchMode()const{
     return inputFileNames.size()>1 || opt_batch_mode;
@@ -330,12 +397,13 @@ string CmdLineOptions::getOutFileSuffix()const{
       case highlight::TEX:
       case highlight::LATEX: return ".tex";
       case highlight::XSLFO: return ".fo";
+      case highlight::XML:   return ".xml";
       default:    return ".html";
     }
 }
 string CmdLineOptions::getDirName(const string & path) {
   size_t dirNameLength=path.rfind(Platform::pathSeparator);
-  return (dirNameLength==string::npos)?string():path.substr(0, dirNameLength);
+  return (dirNameLength==string::npos)?string():path.substr(0, dirNameLength+1);
 }
 bool CmdLineOptions::attachLineAnchors()const{
     return opt_attach_line_anchors;
@@ -355,6 +423,9 @@ bool CmdLineOptions::fopCompatible() const {
 bool CmdLineOptions::replaceQuotes() const {
    return opt_replacequotes;
 }
+bool CmdLineOptions::disableBabelShorthands() const {
+  return opt_babel;
+}
 bool CmdLineOptions::getFlag( const string& paramVal){
    return StringTools::lowerCase(paramVal)=="true";
 }
@@ -367,6 +438,11 @@ bool CmdLineOptions::dataDirGiven()const {
 bool CmdLineOptions::additionalDataDirGiven()const {
   return !additionalDataDir.empty();
 }
+
+bool CmdLineOptions::orderedList() const {
+  return opt_ordered_list;
+}
+
 const string &CmdLineOptions::getDataDir() const {
  return dataDir;
 }
@@ -376,9 +452,12 @@ const string &CmdLineOptions::getIndentScheme() const {
 const string &CmdLineOptions::getAdditionalDataDir()const{
  return additionalDataDir;
 }
-const string &CmdLineOptions::getLanguage()  {
+const string &CmdLineOptions::getLanguage() const {
  return language;
 }
+const string&CmdLineOptions::getEncoding() const{
+    return encodingName;
+}
 bool CmdLineOptions::printIndexFile() const{
  return opt_printindex && (outputType==highlight::HTML ||
                            outputType==highlight::XHTML);
@@ -386,15 +465,22 @@ bool CmdLineOptions::printIndexFile() const{
 bool CmdLineOptions::printProgress() const{
  return opt_print_progress;
 }
-bool CmdLineOptions::fillLineNrZeroes() const{
+bool CmdLineOptions::fillLineNrZeroes() const {
   return opt_fill_zeroes;
 }
-bool CmdLineOptions::syntaxGiven() const{
+bool CmdLineOptions::syntaxGiven() const {
   return opt_language;
 }
-const string  CmdLineOptions::getHelpLang()const{
+bool CmdLineOptions::omitEncoding() const{
+    return StringTools::lowerCase(encodingName)=="none";
+}
+bool CmdLineOptions::forceOutput() const {
+    return opt_force_output;
+}
+const string  CmdLineOptions::getHelpLang() const {
   return helpLang+".help";
 }
+
 highlight::WrapMode CmdLineOptions::getWrappingStyle() const {
   return wrappingStyle;
 }
@@ -412,68 +498,64 @@ void CmdLineOptions::readDirectory(const string & wildcard){
 }
 void CmdLineOptions::loadConfigurationFile()
 {
- string paramVal;
   #ifndef _WIN32
     #ifdef CONFIG_FILE_PATH
-      string configFilePath=CONFIG_FILE_PATH;
+       configFilePath=CONFIG_FILE_PATH;
     #else
       char* homeEnv=getenv("HOME");
       if (homeEnv==NULL) return;
-      string configFilePath=string(homeEnv)+"/.highlightrc";
+      configFilePath=string(homeEnv)+"/.highlightrc";
     #endif
   #else
-    string configFilePath = Platform::getAppPath() + "highlight.conf";
+    configFilePath = Platform::getAppPath() + "highlight.conf";
   #endif
   ConfigurationReader presets(configFilePath);
 
   if (presets.found())
     {
+      string paramVal;
       configFileRead=true;
-      paramVal = presets.getParameter(OPT_CSSOUT);
-      if (!paramVal.empty())
-        {
-          cssOutFilename=paramVal;
-        }
-      paramVal = presets.getParameter(OPT_CSSIN);
-      if (!paramVal.empty())
-        {
-          cssInFilename=paramVal;
-        }
-      paramVal = presets.getParameter(OPT_STYLE);
-      if (!paramVal.empty())
-        {
-          styleName=paramVal;
-          opt_css_style = true;
-        }
-      paramVal = presets.getParameter(OPT_DELTABS);
-      if (!paramVal.empty())
-        {
-          numberSpaces = StringTools::str2int(paramVal);
-        }
-      paramVal = presets.getParameter(OPT_FORMATSTYLE);
-      if (!paramVal.empty())
-        {
-          indentScheme = paramVal;
-        }
-      paramVal = presets.getParameter(OPT_DATADIR);
-      if (!paramVal.empty())
-        {
-          dataDir=validateDirPath(paramVal);
-        }
-      paramVal = presets.getParameter(OPT_ADDDATADIR);
-      if (!paramVal.empty())
-        {
-          additionalDataDir=validateDirPath(paramVal);
-        }
-      paramVal = presets.getParameter(OPT_OUTDIR);
-      if (!paramVal.empty())
-        {
-          outDirectory=validateDirPath(paramVal);
-        }
 
-      opt_include_css=getFlag(presets.getParameter(OPT_INCLUDECSS));
+      styleOutFilename = presets.getParameter(OPT_STYLE_OUT);
+      styleInFilename = presets.getParameter(OPT_STYLE_IN);
+      styleName = presets.getParameter(OPT_STYLE);
+      opt_style = !styleName.empty();
+      language = presets.getParameter(OPT_SYNTAX);
+      opt_language = !language.empty();
+      numberSpaces = StringTools::str2int(presets.getParameter(OPT_DELTABS), true);
+      indentScheme = presets.getParameter(OPT_FORMATSTYLE);
+      baseFontSize = presets.getParameter(OPT_BASE_FONT_SIZE);
+      baseFont = presets.getParameter(OPT_BASE_FONT);
+
+      paramVal = presets.getParameter(OPT_DATADIR);
+      if (!paramVal.empty()) {
+         dataDir=validateDirPath( paramVal);
+      }
+      paramVal = presets.getParameter(OPT_ADDDATADIR);
+      if (!paramVal.empty()) {
+        additionalDataDir=validateDirPath(paramVal);
+      }
+      paramVal = presets.getParameter(OPT_OUTDIR);
+      if (!paramVal.empty()) {
+        outDirectory=validateDirPath(paramVal);
+      }
+      paramVal = presets.getParameter(OPT_ENCODING);
+      if (!paramVal.empty()) {
+          encodingName=paramVal;
+      }
+      paramVal = presets.getParameter(OPT_LNR_WIDTH);
+      if (!paramVal.empty()) {
+          lineNrWidth=StringTools::str2int(string(paramVal), true);
+      }
+      paramVal = presets.getParameter(OPT_LNR_START);
+      if (!paramVal.empty()) {
+          lineNrStart=StringTools::str2int(string(paramVal), true);
+      }
+
+      opt_include_style=getFlag(presets.getParameter(OPT_INC_STYLE));
       opt_verbose=getFlag(presets.getParameter(OPT_VERBOSE));
-      opt_linenumbers=getFlag(presets.getParameter(OPT_LINENO));
+      opt_ordered_list=getFlag(presets.getParameter(OPT_ORDERED_LIST));
+      opt_linenumbers=opt_ordered_list || getFlag(presets.getParameter(OPT_LINENO));
       opt_fragment=getFlag(presets.getParameter(OPT_FRAGMENT));
       opt_attach_line_anchors=getFlag(presets.getParameter(OPT_ANCHORS));
       opt_printindex=getFlag(presets.getParameter(OPT_INDEXFILE));
@@ -483,38 +565,27 @@ void CmdLineOptions::loadConfigurationFile()
       opt_print_progress=getFlag(presets.getParameter(OPT_PROGRESSBAR));
       opt_fill_zeroes=getFlag(presets.getParameter(OPT_FILLZEROES));
 
-      if (getFlag(presets.getParameter(OPT_WRAP)))
-        {
+      if (getFlag(presets.getParameter(OPT_WRAP))) {
           wrappingStyle=highlight::WRAP_DEFAULT;
-        }
-      if (getFlag(presets.getParameter(OPT_WRAPSIMPLE)))
-        {
+      }
+      if (getFlag(presets.getParameter(OPT_WRAPSIMPLE))) {
           wrappingStyle=highlight::WRAP_SIMPLE;
-        }
-      if (getFlag(presets.getParameter(OPT_XHTML)))
-        {
+      }
+      if (getFlag(presets.getParameter(OPT_XHTML))) {
           outputType=highlight::XHTML;
-        }
-      else if (getFlag(presets.getParameter(OPT_RTF)))
-        {
+      } else if (getFlag(presets.getParameter(OPT_RTF))) {
           outputType=highlight::RTF;
-        }
-      else if (getFlag(presets.getParameter(OPT_TEX)))
-        {
+      } else if (getFlag(presets.getParameter(OPT_TEX))) {
           outputType=highlight::TEX;
-        }
-      else if (getFlag(presets.getParameter(OPT_LATEX)))
-        {
+      } else if (getFlag(presets.getParameter(OPT_LATEX))) {
           outputType=highlight::LATEX;
-        }
-      else if (getFlag(presets.getParameter(OPT_XSLFO)))
-        {
+      } else if (getFlag(presets.getParameter(OPT_XSLFO))) {
           outputType=highlight::XSLFO;
-        }
-      else if (getFlag(presets.getParameter(OPT_ANSI)))
-        {
+      } else if (getFlag(presets.getParameter(OPT_ANSI))) {
           outputType=highlight::ANSI;
-        }
+      } else if (getFlag(presets.getParameter(OPT_XML))) {
+            outputType=highlight::XML;
+      }
     }
 }
 
@@ -526,3 +597,37 @@ string CmdLineOptions::validateDirPath(const string & path){
 highlight::OutputType CmdLineOptions::getOutputType() const {
     return outputType;
 }
+
+void CmdLineOptions::printDeprecatedWarning(const char *oldOption, const char *newOption){
+    cerr << "Warning: Long option \""<<oldOption << "\" is DEPRECATED.";
+    cerr << " Use \""<<newOption << "\" instead.\n";
+}
+
+// -- Kafka begins
+
+bool CmdLineOptions::hasBaseFont() const {
+   return (! baseFont.empty()) ;
+}
+
+const string& CmdLineOptions::getBaseFont() const {
+   return baseFont ;
+}
+
+bool CmdLineOptions::hasBaseFontSize() const {
+   return (! baseFontSize.empty()) ;
+}
+
+const string& CmdLineOptions::getBaseFontSize() const {
+   return baseFontSize ;
+}
+
+// -- Kafka ends
+
+
+int CmdLineOptions::getNumberWidth(){
+   return lineNrWidth;
+}
+
+int CmdLineOptions::getNumberStart(){
+   return lineNrStart;
+}
