@@ -2,7 +2,7 @@
                           cmdlineoptions.h  -  description
                              -------------------
     begin                : Sun Nov 25 2001
-    copyright            : (C) 2001 by Andr Simon
+    copyright            : (C) 2001 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -48,7 +48,7 @@
 #endif
 
 #define OPT_VERBOSE      "verbose"
-#define OPT_INCLUDECSS   "include-css"
+#define OPT_INC_STYLE    "include-style"
 #define OPT_HELP         "help"
 #define OPT_LINENO       "linenumbers"
 #define OPT_XHTML        "xhtml"
@@ -65,8 +65,9 @@
 #define OPT_OUT          "output"
 #define OPT_SYNTAX       "syntax"
 #define OPT_STYLE        "style"
-#define OPT_CSSOUT       "css-outfile"
-#define OPT_CSSIN        "css-infile"
+#define OPT_STYLE_OUT    "style-outfile"
+#define OPT_STYLE_IN     "style-infile"
+
 #define OPT_DELTABS      "replace-tabs"
 #define OPT_BATCHREC     "batch-recursive"
 #define OPT_OUTDIR       "outdir"
@@ -84,19 +85,28 @@
 #define OPT_FILLZEROES   "zeroes"
 #define OPT_ANSI         "ansi"
 #define OPT_XML          "xml"
+#define OPT_ENCODING     "encoding"
+#define OPT_FORCE_OUTPUT "force"
+#define OPT_ORDERED_LIST "ordered-list"
+#define OPT_LNR_WIDTH    "line-number-width"
+#define OPT_BASE_FONT    "font"
+#define OPT_BASE_FONT_SIZE "font-size"
+#define OPT_LNR_START    "line-number-start"
+#define OPT_BABEL        "babel"
+
 
 #define S_OPT_ANSI       'A'
 #define S_OPT_OUT        'o'
 #define S_OPT_IN         'i'
 #define S_OPT_SYNTAX     'S'
 #define S_OPT_VERBOSE    'v'
-#define S_OPT_INCLUDECSS 'I'
+#define S_OPT_INC_STYLE  'I'
 #define S_OPT_HELP       'h'
 #define S_OPT_HELPINT    'H'
 #define S_OPT_LINENO     'l'
 #define S_OPT_STYLE      's'
-#define S_OPT_CSSOUT     'c'
-#define S_OPT_CSSIN      'e'
+#define S_OPT_STYLE_OUT  'c'
+#define S_OPT_STYLE_IN   'e'
 #define S_OPT_DELTABS    't'
 #define S_OPT_XHTML      'X'
 #define S_OPT_RTF        'R'
@@ -111,20 +121,32 @@
 #define S_OPT_LISTLANGS  'p'
 #define S_OPT_OUTDIR     'O'
 
-#define S_OPT_FORMATSTYLE 'F'
-#define S_OPT_DATADIR     'D'
-#define S_OPT_ADDDATADIR  'E'
-#define S_OPT_INDEXFILE   'C'
-#define S_OPT_WRAP        'W'
-#define S_OPT_WRAPSIMPLE  'V'
-#define S_OPT_QUIET       'q'
+#define S_OPT_FORMATSTYLE    'F'
+#define S_OPT_DATADIR        'D'
+#define S_OPT_ADDDATADIR     'E'
+#define S_OPT_INDEXFILE      'C'
+#define S_OPT_WRAP           'W'
+#define S_OPT_WRAPSIMPLE     'V'
+#define S_OPT_QUIET          'q'
 #define S_OPT_FOP            'g'
 #define S_OPT_REPLACE_QUOTES 'r'
 #define S_OPT_VERSION        'Q'
 #define S_OPT_PROGRESSBAR    'P'
 #define S_OPT_FILLZEROES     'z'
+#define S_OPT_ENCODING       'u'
+#define S_OPT_LNR_WIDTH      'j'
+#define S_OPT_LNR_START      'm'
+#define S_OPT_BASE_FONT      'k'
+#define S_OPT_BASE_FONT_SIZE 'K'
+#define S_OPT_ORDERED_LIST   'n'
+#define S_OPT_BABEL          'b'
 
-#define S_OPTIONS_STRING "o:i:S:B:O:s:c:e:t:F:D:H:E:afghlvwpqrzACILYRTZXV::W::P"
+// deprecated:
+#define OPT_CSSOUT       "css-outfile"
+#define OPT_CSSIN        "css-infile"
+#define OPT_INC_CSS      "include-css"
+
+#define S_OPTIONS_STRING "o:i:S:B:O:s:c:e:t:u:m:F:D:H:E:abfghlnvwpqrzACILYRTZXUV::W::Pk:K:j:"
 
 using namespace std;
 
@@ -150,11 +172,14 @@ class CmdLineOptions
     /** \return Output directory*/
     const string& getOutDirectory() ;
 
-    /** \return CSS output file name*/
-    const string&getCssOutFilename() const;
+    /** \return Style output file name*/
+    const string getStyleOutFilename() const;
 
-    /** \return CSS input file name*/
-    const string&getCssInFilename() const;
+    /** \return Style input file name*/
+    const string& getStyleInFilename() const;
+
+    /** \return Char set*/
+    const string& getEncoding() const;
 
     /** \return Number of spaces to replace a tab*/
     int getNumberSpaces() const;
@@ -168,8 +193,8 @@ class CmdLineOptions
     /** \return True if debug information should be printed*/
     bool printDebugInfo()const;
 
-    /** \return True if CSS should be included in output*/
-    bool includeCssDefinition() const;
+    /** \return True if Style definition should be included in output*/
+    bool includeStyleDef() const;
 
     /** \return True if line numbers should be printed*/
     bool printLineNumbers() const;
@@ -216,6 +241,9 @@ class CmdLineOptions
     /** \return True if quotes should be replaced by /dq in LaTeX*/
     bool replaceQuotes() const;
 
+    /** \return True if shorthands of LaTeX Babel package should be disabled*/
+    bool disableBabelShorthands() const;
+
     /** \return Data directory*/
     const string &getDataDir()const;
 
@@ -241,7 +269,7 @@ class CmdLineOptions
     const string getHelpLang() const;
 
     /** \return programming language */
-    const string &getLanguage() ;
+    const string &getLanguage()const ;
 
     /** \return Wrapping style*/
     highlight::WrapMode getWrappingStyle() const;
@@ -255,9 +283,45 @@ class CmdLineOptions
     /** \return Output file format */
     highlight::OutputType getOutputType() const;
 
+    /** \return True if chosen output format supports referenced style files */
+    bool formatSupportsExtStyle();
+
+    /** \return True if style output path was defined by user*/
+    bool styleOutPathDefined() const{
+        return opt_stylepath_explicit;
+    }
+
+    /** \return True if encoding specification should be omitted in output*/
+    bool omitEncoding() const;
+
+    /** \return True if output should be generated if languege type is unknown*/
+    bool forceOutput() const;
+
+    /** \return True if line numbers should be replaced by ordered list (HTML) */
+    bool orderedList() const;
+
+    // -- Kafka begins
+    /** \return True if a base font has been given */
+    bool hasBaseFont() const ;
+    /** \return The given base font, empty string by default */
+    const string& getBaseFont() const ;
+    /** \return True if a base font size has been given */
+    bool hasBaseFontSize() const ;
+    /** \return The given base font size, empty string by default */
+    const string& getBaseFontSize() const ;
+    // -- Kafka ends
+
+    /** \return line number width */
+    int getNumberWidth();
+
+    /** \return Line number start count */
+    int getNumberStart();
+
   private:
 
     int numberSpaces;   // number of spaces which replace a tab
+    int lineNrWidth;    // width of line number (left padding)
+    int lineNrStart;    // line number start count
     highlight::WrapMode wrappingStyle; // line wrapping mode
     highlight::OutputType outputType;
 
@@ -270,21 +334,26 @@ class CmdLineOptions
     // name of colour theme
     styleName,
     // name of external style file
-    cssOutFilename,
+    styleOutFilename,
     // name of file to be included in external style file
-    cssInFilename,
+    styleInFilename,
     // used to define data directories at runtime
     dataDir, additionalDataDir;
     // name of indenation scheme
     string indentScheme;
 
+    // -- Kafka begins
+    string baseFont, baseFontSize;
+// -- Kafka ends
+
+
     bool opt_language;
-    bool opt_include_css;
+    bool opt_include_style;
     bool opt_help;
     bool opt_version ;
     bool opt_verbose;
     bool opt_linenumbers;
-    bool opt_css_style;
+    bool opt_style;
     bool opt_batch_mode;
     bool opt_fragment;
     bool opt_attach_line_anchors;
@@ -295,17 +364,18 @@ class CmdLineOptions
     bool opt_quiet;
     bool opt_xslfo_fop;
     bool opt_replacequotes;
+    bool opt_babel;
     bool opt_print_progress;
     bool opt_fill_zeroes;
+    bool opt_stylepath_explicit;
+    bool opt_force_output;
+    bool opt_ordered_list;
 
     bool configFileRead;
 
-    string helpLang;
-
-
+    string helpLang, encodingName;
     string configFilePath;
 
-
     // list of all input file names
     vector <string> inputFileNames;
 
@@ -327,6 +397,7 @@ class CmdLineOptions
     /** \return Valid path name */
     string validateDirPath(const string & path);
 
+    void printDeprecatedWarning(const char *oldOption, const char *newOption);
   };
 
 #endif
