@@ -2,7 +2,7 @@
                           codeparser.cpp  -  description
                              -------------------
     begin                : Die Jul 9 2002
-    copyright            : (C) 2002 by Andr Simon
+    copyright            : (C) 2002 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -24,7 +24,9 @@
 #include "texgenerator.h"
 #include "xslfogenerator.h"
 #include "xmlgenerator.h"
-#ifndef USE_WX_LIBRARY
+#include "re/Matcher.h"
+
+#ifndef __WXMSW__
   #include "ansigenerator.h"
 #endif
 
@@ -35,65 +37,39 @@ namespace highlight {
 
 CodeGenerator* CodeGenerator::generator=NULL;
 
-CodeGenerator* CodeGenerator::getInstance(OutputType type,
-                                          const string& styleInfoPath,
-                                          const string& styleInPath,
-                                          const string& styleOutPath,
-                                          bool includeStyle,
-                                          bool attachAnchors,
-                                          bool replaceQuotes,
-                                          bool fopCompatible,
-                                          int numSpaces,
-                                          WrapMode lineWrappingMode,
-                                          bool ln,
-                                          bool lnz,
-                                          bool fragment
-                                          ) {
+CodeGenerator * CodeGenerator::getInstance(OutputType type) {
   if (generator==NULL){
      switch (type){
+     case HTML:
+        generator = new HtmlGenerator();
+        break;
       case TEX:
-        generator = new TexGenerator (styleInfoPath);
+        generator = new TexGenerator ();
         break;
       case LATEX:
-        generator = new LatexGenerator(styleInfoPath, replaceQuotes);
+        generator = new LatexGenerator();
         break;
       case RTF:
-        generator = new RtfGenerator (styleInfoPath);
+        generator = new RtfGenerator ();
         break;
       case XSLFO:
-        generator = new XslFoGenerator(styleInfoPath, fopCompatible);
+        generator = new XslFoGenerator();
         break;
       case XML:
-        generator = new XmlGenerator(styleInfoPath);
+        generator = new XmlGenerator();
         break;
       case XHTML:
-        generator = new XHtmlGenerator(styleInfoPath,
-                                       styleInPath,
-                                       styleOutPath,
-                                       includeStyle,
-                                       attachAnchors);
+        generator = new XHtmlGenerator();
         break;
-      #ifndef USE_WX_LIBRARY
+      #ifndef __WXMSW__
       case ANSI:
-        generator = new AnsiGenerator (styleInfoPath);
+        generator = new AnsiGenerator ();
         break;
       #endif
       default:
-        generator = new HtmlGenerator(styleInfoPath,
-                                      styleInPath,
-                                      styleOutPath,
-                                      includeStyle,
-                                      attachAnchors);
+        break;
      }
   }
-
-  generator->setPrintLineNumbers(ln);
-  generator->setPrintZeroes(lnz);
-  generator->setFragmentCode(fragment);
-  generator->setPreformatting(lineWrappingMode,
-                             (generator->getPrintLineNumbers()) ?
-                                MAX_LINE__WIDTH - LINE_NUMBER_WIDTH : MAX_LINE__WIDTH,
-                              numSpaces );
   return generator;
 }
 
@@ -102,41 +78,26 @@ void CodeGenerator::deleteInstance(){
   generator=NULL;
 }
 
-
-CodeGenerator::CodeGenerator():
-    in(NULL),
-    out(NULL),
-    maskWs(false),
-    excludeWs(false),
-    fragmentOutput(false),
-    showLineNumbers (false),
-    lineNumberFillZeroes(false),
-    lineNumber(0),
-    lineIndex(0),
-    formatter(NULL),
-    preFormatter(NULL),
-    formattingEnabled(false),
-    formattingPossible(false)
-{}
-
-CodeGenerator::CodeGenerator(const string &colourTheme)
+CodeGenerator::CodeGenerator(highlight::OutputType type)
    :in(NULL),
     out(NULL),
+    encoding("none"),
     maskWs(false),
     excludeWs(false),
     fragmentOutput(false),
     showLineNumbers (false),
     lineNumberFillZeroes(false),
     lineNumber(0),
-    stylePath(colourTheme),
+    lineNumberOffset(0),
+    includeStyleDef(false),
     lineIndex(0),
+    lineNumberWidth(DEFAULT_LINE_NR_WIDTH),
     formatter(NULL),
     preFormatter(NULL),
     formattingEnabled(false),
-    formattingPossible(false)
+    formattingPossible(false),
+    outputType(type)
 {
-  line.reserve(100);
-  docStyle.load(stylePath);
 }
 
 CodeGenerator::~CodeGenerator()
@@ -145,11 +106,47 @@ CodeGenerator::~CodeGenerator()
   delete formatter;
 }
 
-
 /** Getter and Setter*/
 
-void CodeGenerator::setPrintLineNumbers(bool flag){
+void CodeGenerator::setSpecialOptions(bool htmlAttachAnchors,bool htmlOrderedList,
+                           bool latexReplQuotes, bool latexDisableShorthands,
+                           bool xslFopCompatible) {
+   switch (outputType) {
+     case XHTML:
+          ((XHtmlGenerator*)this)->setAttachAnchors(htmlAttachAnchors);
+          ((XHtmlGenerator*)this)->setOrderedList(htmlOrderedList);
+          break;
+     case HTML:
+          ((HtmlGenerator*)this)->setAttachAnchors(htmlAttachAnchors);
+          ((HtmlGenerator*)this)->setOrderedList(htmlOrderedList);
+          break;
+     case XSLFO:
+          ((XslFoGenerator*)this)->setFopCompatible(xslFopCompatible);
+          break;
+     case LATEX:
+          ((LatexGenerator*)this)->setReplaceQuotes(latexReplQuotes);
+          ((LatexGenerator*)this)->setDisableShorthands(latexDisableShorthands);
+     default: break;
+   }
+}
+
+bool CodeGenerator::initStyle(const string& stylePath) {
+  this->stylePath=stylePath;
+  return docStyle.load(stylePath);
+}
+const string& CodeGenerator::getStyleName(){
+  return stylePath;
+}
+
+void CodeGenerator::setLineNumberWidth(int w) {
+  lineNumberWidth=w;
+}
+int CodeGenerator::getLineNumberWidth(){
+  return lineNumberWidth;
+}
+void CodeGenerator::setPrintLineNumbers(bool flag, unsigned int startCnt){
   showLineNumbers=flag;
+  lineNumberOffset = startCnt-1;
 }
 
 bool CodeGenerator::getPrintLineNumbers(){
@@ -164,20 +161,66 @@ bool CodeGenerator::getPrintZeroes(){
   return lineNumberFillZeroes;
 }
 
+void CodeGenerator::setIncludeStyle(bool flag){
+    includeStyleDef = flag;
+}
+
+void CodeGenerator::setStyleInputPath(const string& path){
+    styleInputPath = path;
+}
+void CodeGenerator::setStyleOutputPath(const string& path){
+    styleOutputPath = path;
+}
+
+const string&  CodeGenerator::getStyleInputPath(){
+    return styleInputPath;
+}
+const string&  CodeGenerator::getStyleOutputPath(){
+    return styleOutputPath;
+}
+
 void CodeGenerator::setFragmentCode(bool flag){
   fragmentOutput=flag;
 }
-
 bool CodeGenerator::getFragmentCode(){
   return fragmentOutput;
 }
 
-void CodeGenerator::setStyleName(const string& s){
-  stylePath=s;
+void CodeGenerator::setBaseFont(const string& s) {
+   baseFont = s;
 }
 
-const string& CodeGenerator::getStyleName(){
-  return stylePath;
+void CodeGenerator::setBaseFontSize(const string& s) {
+   baseFontSize = s ;
+}
+
+const string CodeGenerator::getBaseFont() const {
+  if (baseFont.empty()){
+     switch (outputType) {
+        case LATEX:
+          return "ttfamily";
+          break;
+       case TEX:
+          return "tt";
+          break;
+       default:
+          return "Courier";
+     }
+   } else {
+     return baseFont ;
+   }
+}
+
+const string& CodeGenerator::getBaseFontSize()  {
+   if (baseFontSize.empty() && outputType != LATEX && outputType != TEX){
+      return docStyle.getFontSize();
+   } else {
+      return baseFontSize;
+   }
+}
+
+void CodeGenerator::setEncoding(const string& encodingName){
+   encoding = encodingName;
 }
 
 bool CodeGenerator::formattingDisabled(){
@@ -195,19 +238,12 @@ void CodeGenerator::setPreformatting(WrapMode lineWrappingStyle,
   bool replaceTabs = numberSpaces > 0;
   if (enableWrap || replaceTabs)  {
     preFormatter=new PreFormatter(enableWrap, replaceTabs);
-    if (enableWrap)
-      preFormatter->setWrappingProperties(lineLength, lineWrappingStyle==WRAP_DEFAULT);
-    if (replaceTabs)
-      preFormatter->setNumberSpaces(numberSpaces);
+    if (enableWrap) preFormatter->setWrappingProperties(lineLength,
+                                   lineWrappingStyle==WRAP_DEFAULT);
+    if (replaceTabs) preFormatter->setNumberSpaces(numberSpaces);
   }
 }
 
-/*
-WrapMode CodeGenerator::getLineWrapping(){
-  if (preFormatter==NULL) return WRAP_DISABLED;
-  return (preFormatter->indentCode()?WRAP_DEFAULT:WRAP_SIMPLE);
-}
-*/
 LanguageDefinition &CodeGenerator::getLanguage(){
   return langInfo;
 }
@@ -218,7 +254,6 @@ void CodeGenerator::reset()
   line.clear();
 }
 
-
 /** sucht vorwaerts ab Position searchPos Ziffer in s und liefert Integerwert
 der gefundenen Zahl zurueck.
 Im SymbolString stehen die den einzelnen Symbolen zugeordneten Konstanten
@@ -228,6 +263,7 @@ State CodeGenerator::getState(const string &s, unsigned int searchPos)
   unsigned int i= searchPos+1, result=0;
 
   // nach Ziffer in s suchen
+  // TODO replace loops by find methods
   do {
       ++i;
   } while ((i<s.length()) && !isdigit(s[i])) ;
@@ -240,46 +276,6 @@ State CodeGenerator::getState(const string &s, unsigned int searchPos)
   return ((result)? (State)result:_UNKNOWN);
 }
 
-string CodeGenerator::getIdentifier()
-{
-  --lineIndex;
-  unsigned int startPos=lineIndex;
-  char c= line[lineIndex];
-
-  while (    ( lineIndex < line.length()
-          && (   StringTools::isAlpha(c)
-              || isdigit(c))
-              || isAllowedChar(c))
-          )
-    {
-      ++lineIndex;
-      c= line[lineIndex];
-    }
-  return string(line, startPos, lineIndex - startPos);
-}
-
-string CodeGenerator::getNumber()
-{
-  --lineIndex;
-  unsigned int startPos=lineIndex;
-  char c=line[lineIndex];
-
-  while ( lineIndex < line.length() && (
-          isdigit(c)
-          // don't highlight methods applied on numbers as part of the number
-          // i.e. Ruby: 3.xxx()
-          || (c == '.' && isdigit(line[lineIndex+1]))
-          // '-' is accepted as first character
-          || (c == '-' && lineIndex == startPos)
-          || (StringTools::isAlpha(c) && line[lineIndex-1]=='0')
-          || (isxdigit(c) || c=='L' || c=='U' || c=='l' || c=='u') ))
-    {
-      ++lineIndex;
-      c= line[lineIndex];
-    }
-  return string(line,startPos, lineIndex-startPos);
-}
-
 unsigned int CodeGenerator::getLineNumber()
 {
   return lineNumber;
@@ -288,21 +284,41 @@ unsigned int CodeGenerator::getLineNumber()
 bool CodeGenerator::readNewLine(string &newLine){
   bool eof;
   terminatingChar=newLine[lineIndex-1];
-  if (formattingPossible && formattingEnabled)
-   {
+  if (formattingPossible && formattingEnabled) {
      eof=!formatter->hasMoreLines();
-      if (!eof)
-       {
+     if (!eof) {
          newLine = formatter->nextLine();
-       }
+     }
+   }
+   // reformatting not enabled
+   else {
+     eof = ! getline( *in, newLine);
    }
-   else  // reformatting not enabled
-    {
-      eof = ! getline( *in, newLine);
-    }
    return eof;
 }
 
+void CodeGenerator::matchRegex(const string &line){
+    map <int, ReGroup*>::iterator i;
+    for(i = regexGroups.begin(); i != regexGroups.end(); i++) {
+        delete i->second;
+        i->second = 0;
+    }
+    regexGroups.clear();
+
+    // cycle through all regex, save the start and ending indices to report them later
+    for (unsigned int i=0; i<langInfo.getRegexElements().size(); i++){
+        RegexElement *regexElem = langInfo.getRegexElements()[i];
+        Matcher * matcher = regexElem->rePattern->createMatcher(line);
+        while (matcher->findNextMatch()) {
+            regexGroups.insert( make_pair(matcher->getStartingIndex(matcher->getGroupNum()-1)+1,
+                                          new ReGroup(regexElem->open,
+                                                      matcher->getGroup(matcher->getGroupNum()-1).size(),
+                                                      regexElem->kwClass)) );
+        }
+        delete matcher;
+   }
+}
+
 unsigned char CodeGenerator::getInputChar()
 {
   bool eol = lineIndex == line.length();
@@ -321,22 +337,26 @@ unsigned char CodeGenerator::getInputChar()
     lineIndex=0;
     ++lineNumber;
     line=StringTools::trimRight(line);
+    matchRegex(line);
+
     return (eof)?'\0':'\n';
   }
+
   return line[lineIndex++];
 }
 
 State CodeGenerator::getCurrentState (bool lastStateWasNumber)
 {
+
   unsigned char c;
 
   if (token.length()==0) {
     c=getInputChar();
+
   }  else {
     lineIndex-= (token.length()-1);
     c=token[0];
   }
-
   if (c=='\n'){
     return _EOL;   // End of line
   }
@@ -345,30 +365,36 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
     return _EOF;   // End of file
   }
 
-  if (isspace(c)) {
-        // keep original tabs if not replaced with -t option
-        /*if (c=='\t'){
-          token= c;
-        } else {
-          token= spacer;
-        }*/
+  if (iswspace(c)) {
       token= c;
       return _WS;
-    }
+  }
 
-  // numbers have to be searched before using the symbolstring,
-  // as numbers are part of this string
-  if (isdigit(c)
-      // recognize floats like .5
-      || (c=='.' && isdigit(line[lineIndex]))
-      // test if '-' belongs to a term like "1-2"
-      || ((c == '-')
-          && (!lastStateWasNumber)
-          && isdigit(StringTools::getNextNonWs(line, lineIndex))) )
-    {
-      token = getNumber();
-      return NUMBER;
-    }
+  // Test if a regular expression was found at the current position
+  if (!regexGroups.empty()){
+      if (regexGroups.count(lineIndex)) {
+       token = line.substr(lineIndex-1, regexGroups[lineIndex]->length);
+       unsigned int oldIndex= lineIndex;
+       if (regexGroups[oldIndex]->length>1) lineIndex+= regexGroups[lineIndex]->length-1;
+
+       // TODO join code....
+       if (regexGroups[oldIndex]->state==IDENTIFIER_BEGIN) {
+           string reservedWord=(langInfo.isIgnoreCase()) ?
+                   StringTools::lowerCase(token):token;
+           currentKeywordClass=langInfo.isKeyword(reservedWord);
+           return (currentKeywordClass) ? KEYWORD : STANDARD;
+       }
+       else if (regexGroups[oldIndex]->state==KEYWORD) {
+           string reservedWord=(langInfo.isIgnoreCase()) ?
+                   StringTools::lowerCase(token):token;
+           int isListedKeyword = langInfo.isKeyword(reservedWord);
+           currentKeywordClass=(isListedKeyword)? isListedKeyword : regexGroups[oldIndex]->kwClass;
+           return KEYWORD;
+       } else {
+           return regexGroups[oldIndex]->state;
+        }
+     }
+  }
 
   unsigned int symbolLength;
   size_t symbolPos;
@@ -383,16 +409,10 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
       symbolLength=symbols.find(' ', symbolPos)-symbolPos;
       token = symbols.substr(symbolPos, symbolLength);
 
-      // TODO Ruby =ende, =end bugfix (whitespace after symbol needs to be checked)
-
       // Abfrage nach Leerzeichen in SymbolString verhindert falsches
       // Erkennen von Symbolteilen:
       if (lineIndex && token == line.substr(lineIndex-1, symbolLength)
-                    //&& symbols[symbolPos-1] == ' '
-                    && isspace(symbols[symbolPos-1])
-                //    && isspace(line[lineIndex + symbolLength+1])
-
-          ) {
+                    && iswspace(symbols[symbolPos-1]) ) {
          found = true;
          lineIndex += (symbolLength-1);
       } else {
@@ -405,15 +425,12 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
     State foundState = getState(symbols, symbolPos);
 
     // get the current keyword class id to apply the corresponding formatting style
-    if (foundState==KEYWORD_BEGIN || foundState==TAG_BEGIN ) {
+    if (foundState==TAG_BEGIN ) {
       currentKeywordClass=langInfo.getDelimPrefixClassID(token);
     }
-
-    // Full line quotes must start in coloumn 1 (Fortran 77)
-    if (langInfo.isFullLineComment() && foundState==SL_COMMENT){
-      if (lineIndex==1) {
-        return SL_COMMENT;
-      }
+    // simple one byte escape characters:
+    else if (foundState==ESC_CHAR && lineIndex<line.length()) {
+        if (token.length()==1) token += line[lineIndex++];
     }
     // VHDL Workaround: distinguish string delimiters and event markers
     // (same eymbol: ')
@@ -421,37 +438,10 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
              && foundState==STRING && currentState!=STRING
              && lineIndex > 1
              &&(isdigit(line[lineIndex-2]) || isalpha(line[lineIndex-2]))){
-        c=line[lineIndex-1];
-        // do not return, continue search...
-    }  else {
-        return foundState;
-    }
+        return STANDARD;
+  }
+    return foundState;
   }
-
-  // Alphanumerisches Token parsen und als Keyword oder Type erkennen
-  if (StringTools::isAlpha(c) || langInfo.isPrefix(c) || isAllowedChar(c))
-    {
-      if (langInfo.isPrefix(c)){
-        token = c;
-        ++lineIndex;
-        token += getIdentifier();
-      } else {
-        token = getIdentifier();
-      }
-      string reservedWord=(langInfo.isIgnoreCase()) ?
-                          StringTools::lowerCase(token):token;
-      /*int newKeywordClass=langInfo.isKeyword(reservedWord);
-      if (newKeywordClass){
-          currentKeywordClass = newKeywordClass;
-          return KEYWORD;
-      } else {
-          return STANDARD;
-      }*/
-
-      currentKeywordClass=langInfo.isKeyword(reservedWord);
-      return (currentKeywordClass) ? KEYWORD : STANDARD;
-
-    }
 
   // Character not referring to any state
   token = c;
@@ -469,24 +459,15 @@ string CodeGenerator::maskString(const string & s)
 
 void CodeGenerator::printMaskedToken(bool flushWhiteSpace)
 {
-  if(flushWhiteSpace) flushWs();
+  if (flushWhiteSpace) flushWs();
   *out << maskString(token);
   token.clear();
 }
 
-bool CodeGenerator::isAllowedChar(char c)
-{
-  return ( langInfo.getAllowedChars().find(c)!= string::npos);
-}
-
 bool CodeGenerator::styleFound(){
   return docStyle.found();
 }
 
-bool CodeGenerator::printExternalStyle(const string &outFile){
- return true;
-}
-
 bool CodeGenerator::printIndexFile(const vector<string> &fileList,
                                    const string &outPath){
   return true;
@@ -668,21 +649,22 @@ LoadResult CodeGenerator::initLanguage(const string& langDefPath){
        styleTagOpen.erase(keyStyleOpenBegin, styleTagOpen.end());
        styleTagClose.erase(keyStyleCloseBegin, styleTagClose.end());
     }
-    // add new keyword delimiters
+    // add new keyword tag delimiters
     for (unsigned int i=0;i< langInfo.getKeywordClasses().size(); i++){
       styleTagOpen.push_back(getMatchingOpenTag(i));
       styleTagClose.push_back(getMatchingCloseTag(i));
     }
   }
-  return (reloadNecessary)?LOAD_NEW:LOAD_NONE;
+  return (reloadNecessary) ? LOAD_NEW : LOAD_NONE;
 }
 
-ParseError CodeGenerator::printOutput (const string & inFileName,
+ParseError CodeGenerator::generateFile (const string &inFileName,
                                        const string &outFileName)
 {
   if (!docStyle.found()) {
     return BAD_STYLE;
   }
+
   reset();
 
   ParseError error=PARSE_OK;
@@ -699,7 +681,6 @@ ParseError CodeGenerator::printOutput (const string & inFileName,
      error=BAD_INPUT;
   }
 
-
   if (error==PARSE_OK) {
     if (formatter != NULL){
        formatter->init(new astyle::ASStreamIterator(in));
@@ -707,7 +688,9 @@ ParseError CodeGenerator::printOutput (const string & inFileName,
     if (! fragmentOutput){
       *out << getHeader(inFileName);
     }
+
     printBody();
+
     if (! fragmentOutput){
       *out << getFooter();
     }
@@ -722,10 +705,49 @@ ParseError CodeGenerator::printOutput (const string & inFileName,
   return error;
 }
 
+string CodeGenerator::generateString(const string &input, const string &title){
+
+  if (!docStyle.found()) {
+    return "";
+  }
+
+  reset();
+
+  in = new istringstream (input);
+  out = new ostringstream ();
+
+  if ( in->fail() || out->fail()){
+     return "";
+  }
+
+  if (formatter != NULL){
+       formatter->init(new astyle::ASStreamIterator(in));
+  }
+  if (! fragmentOutput){
+    *out << getHeader(title);
+  }
+
+  printBody();
+
+  if (! fragmentOutput){
+    *out << getFooter();
+  }
+
+  string result = static_cast<ostringstream*>(out)->str();
+
+  delete out; out=NULL;
+  delete in; in=NULL;
+
+  return result;
+}
+
+const char* CodeGenerator::generateCString(const string &input, const string &title) {
+  return generateString(input, title).c_str();
+}
 
 unsigned int CodeGenerator::getStyleID(State s, unsigned int kwClassID){
-    if (s==KEYWORD && kwClassID){
-      return NUMBER_BUILTIN_STYLES + kwClassID-1;
+  if (s==KEYWORD && kwClassID){
+    return NUMBER_BUILTIN_STYLES + kwClassID-1;
   }
   return (unsigned int) s ;
 }
@@ -757,6 +779,7 @@ void CodeGenerator::openKWTag(unsigned int kwClassID){
 
 void CodeGenerator::processRootState()
 {
+
   if (langInfo.highlightingDisabled()){
      string line;
      while (getline(*in, line)){
@@ -774,11 +797,11 @@ void CodeGenerator::processRootState()
   do {
     // determine next state
     state= getCurrentState(state==NUMBER);
+
     // handle current state
     switch(state)
       {
       case KEYWORD:
-      case KEYWORD_BEGIN:
         closeTag(STANDARD);
         eof=processKeywordState(state);
         openTag(STANDARD);
@@ -788,7 +811,7 @@ void CodeGenerator::processRootState()
         eof=processNumberState();
         openTag(STANDARD);
         break;
-      case ML_COMMENT_BEGIN:
+      case ML_COMMENT:
         closeTag(STANDARD);
         eof=processMultiLineCommentState();
         openTag(STANDARD);
@@ -803,7 +826,7 @@ void CodeGenerator::processRootState()
         eof=processStringState(STANDARD);
         openTag(STANDARD);
         break;
-      case DIRECTIVE_LINE:
+      case DIRECTIVE:
         closeTag(STANDARD);
         eof=processDirectiveState();
         openTag(STANDARD);
@@ -813,10 +836,14 @@ void CodeGenerator::processRootState()
         eof=processTagState();
         openTag(STANDARD);
         break;
-      case ESC_CHAR_EXT:
-        closeTag(STANDARD);
-        eof=processEscapeCharState();
-        openTag(STANDARD);
+      case ESC_CHAR:
+        if (langInfo.allowExtEscSeq()){
+          closeTag(STANDARD);
+          eof=processEscapeCharState();
+          openTag(STANDARD);
+        } else {
+          printMaskedToken();
+        }
         break;
       case SYMBOL:
         closeTag(STANDARD);
@@ -850,7 +877,6 @@ bool CodeGenerator::processKeywordState(State myState){
   bool eof=false,
        exitState=false;
 
-  //openTag(KEYWORD);
    openKWTag(myClassID);
   do {
     printMaskedToken(newState!=_WS);
@@ -862,24 +888,20 @@ bool CodeGenerator::processKeywordState(State myState){
         break;
       case _EOL:
         insertLineNumber();
+        exitState=true;
         break;
       case _EOF:
         eof = true;
         break;
       case KEYWORD_END:
-         if (myState==KEYWORD_BEGIN){
-           printMaskedToken();
-         }
          exitState=true;
         break;
       default:
-        exitState=    myState!=KEYWORD_BEGIN
-                   &&((myClassID!=currentKeywordClass)||(myState!=newState));
+        exitState= (myClassID!=currentKeywordClass)||(myState!=newState);
         break;
       }
   } while ((!exitState) && (!eof));
 
-  //closeTag(KEYWORD);
   closeKWTag(myClassID);
 
   currentKeywordClass=0;
@@ -890,7 +912,6 @@ bool CodeGenerator::processNumberState(){
   State newState=STANDARD;
   bool eof=false,
        exitState=false;
-
   openTag(NUMBER);
   do {
     printMaskedToken(newState!=_WS);
@@ -902,6 +923,7 @@ bool CodeGenerator::processNumberState(){
         break;
       case _EOL:
         insertLineNumber();
+        exitState=true;
         break;
       case _EOF:
         eof = true;
@@ -921,8 +943,7 @@ bool CodeGenerator::processMultiLineCommentState()
   int commentCount=1;
   State newState=STANDARD;
   bool eof=false, exitState=false;
-
-  openTag(ML_COMMENT_BEGIN);
+  openTag(ML_COMMENT);
   do {
     printMaskedToken(newState!=_WS);
     newState= getCurrentState();
@@ -933,12 +954,14 @@ bool CodeGenerator::processMultiLineCommentState()
         processWsState();
         break;
       case _EOL:
+        wsBuffer += styleTagClose[ML_COMMENT];
         insertLineNumber();
+        wsBuffer += styleTagOpen[ML_COMMENT];
         break;
       case _EOF:
         eof = true;
         break;
-      case ML_COMMENT_BEGIN:
+      case ML_COMMENT:
         if (langInfo.allowNestedMLComments()) {
               ++commentCount;
         }
@@ -955,12 +978,15 @@ bool CodeGenerator::processMultiLineCommentState()
       }
   } while ((!exitState) && (!eof));
 
-  closeTag(ML_COMMENT_BEGIN);
+  closeTag(ML_COMMENT);
   return eof;
 }
 
 bool CodeGenerator::processSingleLineCommentState()
 {
+
+  //if ( checkSpecialCmd()) return false;
+
   State newState=STANDARD;
   bool eof=false, exitState=false;
 
@@ -976,8 +1002,8 @@ bool CodeGenerator::processSingleLineCommentState()
         break;
      case _EOL:
         printMaskedToken();
-        exitState=true;
         insertLineNumber();
+        exitState=true;
         break;
       case _EOF:
         eof = true;
@@ -996,7 +1022,7 @@ bool CodeGenerator::processDirectiveState()
   State  newState=STANDARD;
   bool eof=false, exitState=false;
 
-  openTag(DIRECTIVE_LINE);
+  openTag(DIRECTIVE);
   do {
     printMaskedToken(newState!=_WS);
     newState= getCurrentState();
@@ -1005,30 +1031,32 @@ bool CodeGenerator::processDirectiveState()
       case _WS:
         processWsState();
         break;
-      case DIRECTIVE_LINE_END:
+      case DIRECTIVE_END:
         printMaskedToken();
         exitState=true;
         break;
       case _EOL:
         printMaskedToken();
         exitState=(terminatingChar!=langInfo.getContinuationChar());
+        if (!exitState) wsBuffer += styleTagClose[DIRECTIVE];
         insertLineNumber();
+        if (!exitState) wsBuffer += styleTagOpen[DIRECTIVE];
         break;
-      case ML_COMMENT_BEGIN:
-        closeTag(DIRECTIVE_LINE);
+      case ML_COMMENT:
+        closeTag(DIRECTIVE);
         eof= processMultiLineCommentState();
-        openTag(DIRECTIVE_LINE);
+        openTag(DIRECTIVE);
         break;
       case SL_COMMENT:
-        closeTag(DIRECTIVE_LINE);
+        closeTag(DIRECTIVE);
         eof= processSingleLineCommentState();
-        openTag(DIRECTIVE_LINE);
+        openTag(DIRECTIVE);
         exitState=true;
         break;
       case STRING:
-        closeTag(DIRECTIVE_LINE);
-        eof=processStringState(DIRECTIVE_LINE);
-        openTag(DIRECTIVE_LINE);
+        closeTag(DIRECTIVE);
+        eof=processStringState(DIRECTIVE);
+        openTag(DIRECTIVE);
         break;
       case _EOF:
         eof = true;
@@ -1038,7 +1066,7 @@ bool CodeGenerator::processDirectiveState()
       }
   } while ((!exitState) && (!eof));
 
-  closeTag(DIRECTIVE_LINE);
+  closeTag(DIRECTIVE);
   return eof;
 }
 
@@ -1050,11 +1078,11 @@ bool CodeGenerator::processStringState(State oldState)
   // Test if character before string open delimiter token equals to the
   // raw string prefix (Example: r" ", r""" """ in Python)
   bool isRawString=
-         line[lineIndex-token.length()-1]==langInfo.getRawStringPrefix();
+       line[lineIndex-token.length()-1]==langInfo.getRawStringPrefix();
 
   string openStringDelimiter=token;
 
-  State myState= (oldState==DIRECTIVE_LINE) ? DIRECTIVE_STRING : STRING;
+  State myState= (oldState==DIRECTIVE) ? DIRECTIVE_STRING : STRING;
   openTag(myState);
   do {
     // true if last token was an escape char
@@ -1070,11 +1098,14 @@ bool CodeGenerator::processStringState(State oldState)
         processWsState();
         break;
       case _EOL:
+        wsBuffer += styleTagClose[myState];
         insertLineNumber();
+        wsBuffer += styleTagOpen[myState];
         break;
-      case ML_COMMENT_END:
-        printMaskedToken();
-        break;
+      case STRING_END:
+          exitState= true;
+          printMaskedToken();
+          break;
       case STRING:
         exitState= openStringDelimiter==token;
         printMaskedToken();
@@ -1121,6 +1152,7 @@ bool CodeGenerator::processTagState()
         break;
       case _EOL:
         insertLineNumber();
+        exitState=true;
         break;
       case TAG_END:
         printMaskedToken();
@@ -1179,6 +1211,7 @@ bool CodeGenerator::processSymbolState(){
         break;
       case _EOL:
         insertLineNumber();
+        exitState=true;
         break;
       case _EOF:
         eof = true;
@@ -1197,27 +1230,23 @@ bool CodeGenerator::processEscapeCharState()
 {
   State newState=STANDARD;
   bool eof=false, exitState=false;
-
   openTag(ESC_CHAR);
   do {
     printMaskedToken(newState!=_WS);
-    skipEscapeSequence();
     newState= getCurrentState();
-    switch(newState)
-      {
+    switch(newState) {
       case _EOL:
         insertLineNumber();
         exitState=true;
         break;
       case _WS:
         processWsState();
-        --lineIndex;
         break;
       case _EOF:
         eof = true;
         break;
       default:
-        exitState=newState!=ESC_CHAR && newState!=ESC_CHAR_EXT;
+        exitState=newState!=ESC_CHAR;
         break;
       }
   } while ((!exitState) && (!eof));
@@ -1226,28 +1255,6 @@ bool CodeGenerator::processEscapeCharState()
   return eof;
 }
 
-void  CodeGenerator::skipEscapeSequence(){
-  if (lineIndex<line.length()){
-    char c=line[lineIndex];
-    int charsToSkip=1;
-    // Escape Sequenz /ooo Oktal, /x000 hex, /u00xx Java unicode
-    if (isdigit(c) ){
-      // \0 abfangen
-      while ( isdigit(line[lineIndex+charsToSkip]) && charsToSkip<4) {
-        ++charsToSkip;
-      }
-    } else if (tolower(c)=='x'){
-      charsToSkip=langInfo.isJava() ? 4 : 3;
-    } else if (tolower(c)=='u'){
-      charsToSkip=5;
-    }
-    while (charsToSkip-- && lineIndex++<line.length()){
-       *out <<maskCharacter(line[lineIndex-1]);
-    }
-  }
-}
-
-
 void CodeGenerator::processWsState()
 {
   if (!maskWs) {
@@ -1255,13 +1262,13 @@ void CodeGenerator::processWsState()
     token.clear();
     return;
   }
-
+  flushWs();
   int cntWs=0;
   lineIndex--;
 
-   while (isspace(line[lineIndex])  ) {
-    ++cntWs;
-    ++lineIndex;
+   while (iswspace(line[lineIndex])  ) {
+     ++cntWs;
+     ++lineIndex;
   }
 
   if (cntWs>1) {
@@ -1291,7 +1298,7 @@ void CodeGenerator::flushWs(){
 bool CodeGenerator::isFirstNonWsChar() {
   unsigned int i=lineIndex-1;
   while (i--){
-    if (!isspace(line[i])){
+    if (!iswspace(line[i])){
       return false;
     }
   }
@@ -1303,33 +1310,103 @@ string CodeGenerator::getNewLine(){
 }
 
 void CodeGenerator::insertLineNumber(bool insertNewLine) {
-  if (insertNewLine){
-    *out << getNewLine();
+    if (insertNewLine){
+    wsBuffer += getNewLine();
   }
   if (showLineNumbers) {
     ostringstream os;
+    ostringstream numberPrefix;
     if (lineNumberFillZeroes) {
       os.fill('0');
     }
-    os <<setw(LINE_NUMBER_WIDTH) << right << lineNumber;
+    os <<setw(getLineNumberWidth()) << right << lineNumber+lineNumberOffset;
 
-    unsigned int styleID=getStyleID(currentState, currentKeywordClass);
+    numberPrefix << styleTagOpen[LINENUMBER]
+                 << maskString(os.str()) << spacer
+                 << styleTagClose[LINENUMBER];
 
-    if (styleID!=_UNKNOWN){
-      *out << styleTagClose[styleID];
-    }
-    *out << styleTagOpen[LINENUMBER]
-         << maskString(os.str()) << spacer
-         << styleTagClose[LINENUMBER];
-
-    if (styleID!=_UNKNOWN){
-      *out << styleTagOpen[styleID];
-    }
+    wsBuffer += numberPrefix.str();
   }
 }
 
-unsigned int  CodeGenerator::getLineIndex(){
+unsigned int CodeGenerator::getLineIndex(){
   return lineIndex;
 }
 
+bool CodeGenerator::printExternalStyle(const string &outFile)
+{
+    if (!includeStyleDef && langInfo.getSyntaxHighlight()) {
+        ofstream cssOutFile(outFile.c_str());
+        if (cssOutFile) {
+            cssOutFile << styleCommentOpen
+                       <<" Style definition file generated by highlight "
+                       << HIGHLIGHT_VERSION << ", " << HIGHLIGHT_URL
+                       << " " << styleCommentClose << "\n";
+            cssOutFile << "\n" << styleCommentOpen
+                       << " Highlighting theme definition: "
+                       << styleCommentClose << "\n\n"
+                       << getStyleDefinition()
+                       << "\n";
+            cssOutFile << readUserStyleDef();
+            cssOutFile.close();
+        } else {
+            return false;
+        }
+    }
+    return true;
+}
+
+string CodeGenerator::readUserStyleDef(){
+    ostringstream ostr;
+    if (!styleInputPath.empty()){
+        ifstream userStyleDef(styleInputPath.c_str());
+        if (userStyleDef) {
+            ostr << "\n" << styleCommentOpen
+                 << " Content of " << styleInputPath
+                 << ": " <<styleCommentClose << "\n";
+            string line;
+            while (getline(userStyleDef, line)){
+                ostr << line << "\n";
+            }
+            userStyleDef.close();
+        } else {
+            ostr << styleCommentOpen
+                 << " ERROR: Could not include " << styleInputPath
+                 << "." << styleCommentClose << "\n";
+        }
+    }
+    return ostr.str();
+}
+
+bool CodeGenerator::checkSpecialCmd(){
+    bool insertNL = (lineIndex-token.length());
+    //cerr << "token: "<<token<< " index"<< lineIndex << " "<<line [ lineIndex ]<<endl;
+
+    if (line [ lineIndex ]=='!'){
+      // find cmd
+      size_t cmdPos1 = line.find('@', lineIndex+1);
+
+      cerr << "cmdPos"<<cmdPos1<<endl;
+      if(cmdPos1==string::npos) return false;
+      size_t cmdPos2=cmdPos1+1;
+      while (cmdPos2 < line.length() && StringTools::isAlpha(line[cmdPos2])) cmdPos2++;
+      cerr << "cmdPos2"<<cmdPos2<<endl;
+      cerr << line.substr(cmdPos1, cmdPos2)<<endl;
+
+      // hide comment line
+      token.clear();
+      lineIndex=line.length();
+      getInputChar(); lineNumber--;
+      if (insertNL) {
+        lineNumber++;
+        insertLineNumber();
+      }
+      // end hide
+
+      return true;
+    }
+
+    return false;
+}
+
 }
