@@ -22,33 +22,33 @@
 #include <sstream>
 #include <string>
 #include <iomanip>
-#include <cctype>
+
+// Avoid problems with isspace and UTF-8 characters, use iswspace instead
+//#include <cctype>
+#include <wctype.h>
 
 #include "languagedefinition.h"
 #include "documentstyle.h"
-#include "ASFormatter.h"
+#include "astyle/ASFormatter.h"
 #include "preformatter.h"
 #include "enums.h"
 
 
 #define NUMBER_BUILTIN_STYLES 10
-#define LINE_NUMBER_WIDTH     5
-#define MAX_LINE__WIDTH       80
-
-#define OUTPUT_FLAG_LN    1
-#define OUTPUT_FLAG_LNZ   2
-#define OUTPUT_FLAG_FRAG  4
+#define DEFAULT_LINE_NR_WIDTH     5
 
 /** The highlight namespace contains all classes and data structures
     needed for parsing input data.
 */
 namespace highlight {
 
+class ReGroup;
+
 /** \brief Base class for parsing. Works similar to a Turing machine.
 
-    The virtual class provides source code parsing functioality, based on
+    The virtual class provides source code parsing functionality, based on
     information stored in language definitions.<br>
-    Deriving classes have to define the output format.<br>
+    The derived classes have to define the output format.<br>
     Codegenerator is a singleton class.
 
 * @author Andre Simon
@@ -64,32 +64,9 @@ class CodeGenerator
     /**
       Get appropriate Codegenerator instance
       \param type Output file type (HTML, XHTML, RTF, LATEX, TEX, XSLFO, ANSI)
-      \param styleInfoPath Path to formatting style information
-      \param styleInPath Path to style definition input file (to be included in styleOutPath)
-      \param styleOutPath Path to style definition output file (CSS path for HTML output)
-      \param includeStyle Switch to include style information in output file (only XHTML, HTML)
-      \param attachAnchors Switch to attach anchors to line numbers (only XHTML, HTML)
-      \param replaceQuotes Switch to replace quotes by \dq{} (only LATEX)
-      \param fopCompatible Switch to generate FO for Apache FOP  (only XSLFO)
-      \param ln Set true if line numbers should be printed
-      \param lnz Set true if leading space of line numbers should be filled with 0's
-      \param fragment Set true if document header and footer should be omitted
-      \param numSpaces Number of spaces which replace a tab
-      \param lineWrappingMode Line wrapping mode
     */
-    static CodeGenerator* getInstance(OutputType type,
-                                      const string& styleInfoPath,
-                                      const string& styleInPath,
-                                      const string& styleOutPath,
-                                      bool includeStyle,
-                                      bool attachAnchors,
-                                      bool replaceQuotes,
-                                      bool fopCompatible,
-                                      int numSpaces,
-                                      WrapMode lineWrappingMode,
-                                      bool ln,
-                                      bool lnz,
-                                      bool fragment );
+    static CodeGenerator* getInstance(OutputType type);
+
 
     /** Deletes the singleton CodeGenerator instance.
         Call this method if getInstance was already called, or if you want to
@@ -97,17 +74,39 @@ class CodeGenerator
     static void deleteInstance();
 
     /**
-     Generates output
+     Generates output file
      \param inFileName Path of input file (if empty use stdin)
-     \param outFileName Path of input file (if empty use stdout)
-
+     \param outFileName Path of output file (if empty use stdout)
      \return ParseError
     */
-    ParseError printOutput(const string &inFileName, const string &outFileName);
+    ParseError generateFile(const string &inFileName, const string &outFileName);
+
+    /**
+     Generates output string
+     \param input input code
+     \param title document title
+     \return formatted output code
+    */
+    string generateString(const string &input, const string &title="");
+
+    /**
+     Generates output string (added to make SWIG integration easier)
+     \param input input code
+     \param title document title
+     \return formatted output code
+    */
+    const char * generateCString(const string &input, const string &title="");
 
     /** \return True if document style was found */
     bool styleFound();
 
+    /**
+     Define colour style information
+     \param stylePath Path of style description file
+     \return true if successfull
+    */
+    bool initStyle(const string& stylePath);
+
     /** \return True if reformatting of current input is disabled */
     bool formattingDisabled();
 
@@ -124,13 +123,13 @@ class CodeGenerator
 
     /** tell parser to output line numbers
        \param  flag true if line numbers should be printed
+       \param startCnt line number starting count
     */
-    void setPrintLineNumbers(bool flag);
+    void setPrintLineNumbers(bool flag, unsigned int startCnt=0);
 
     /** \return line number flag */
     bool getPrintLineNumbers();
 
-
     /** tell parser to output line numbers filled with zeroes
         \param  flag true if zeroes should be printed
     */
@@ -144,17 +143,40 @@ class CodeGenerator
     */
     void setFragmentCode(bool flag);
 
+    /** define line number width
+       \param  w width
+    */
+    void setLineNumberWidth(int w);
+
+    /** \return line number width */
+    int getLineNumberWidth();
+
     /** \return fragment flag */
     bool getFragmentCode();
 
-    /** tell parser the style name
-       \param s path to style definition
-    */
-    void setStyleName(const string& s);
-
     /** \return style path */
     const string& getStyleName();
 
+    // -- Kafka begins
+
+    /** tell parser to use this font as base font
+    \param s the font name, e.g. "Courier New"
+     */
+    void setBaseFont(const string& s);
+
+    /** \return base font */
+    const string getBaseFont() const ;
+
+    /** tell parser to use this size as base font size
+    \param s the font size, e.g. "12"
+     */
+    void setBaseFontSize(const string& s);
+
+    /** \return base font size*/
+    const string& getBaseFontSize() ;
+
+// -- Kafka ends
+
     /** tell parser the wrapping mode
        \param lineWrappingStyle wrapping style
        \param lineLength max line length
@@ -162,13 +184,29 @@ class CodeGenerator
     */
     void setPreformatting(WrapMode lineWrappingStyle, unsigned int lineLength,int numberSpaces);
 
-    /** \return wrapping style */
-    WrapMode getLineWrapping();
+//   /** \return wrapping style */
+//    WrapMode getLineWrapping();
+
+    /** tell parser the include style definition in output
+        \param flag true if style should be included
+     */
+    void setIncludeStyle(bool flag);
+
+    /** define output format specific options
+        \param htmlAttachAnchors set true if anchors should be added to line numbers
+        \param htmlOrderedList set true if line numbers should be replaced by ordered list
+        \param latexReplQuotes set true if quotes should be replaced by \dq
+        \param latexDisableShorthands set true if Babel shorthands should be disabled
+        \param xslFopCompatible set true if XSL output should be compatible with Apache FOP
+     */
+    void setSpecialOptions(bool htmlAttachAnchors,bool htmlOrderedList,
+                           bool latexReplQuotes, bool latexDisableShorthands,
+                           bool xslFopCompatible);
 
    /** Print style definitions to external file
      \param outFile Path of external style definition
      */
-    virtual bool printExternalStyle(const string &outFile);
+    bool printExternalStyle(const string &outFile);
 
    /** Print index file with all input file names
       \param fileList List of output file names
@@ -183,15 +221,30 @@ class CodeGenerator
     */
     bool initIndentationScheme(const string&indentSchemePath);
 
-protected:
+    /** Set style input path
+       \param path path to style input file
+     */
+    void setStyleInputPath(const string& path);
 
-    CodeGenerator();
-
-    //! CodeGenerator Constructor
-    /**
-        \param colourTheme Name of coloring style being used
+    /** Set style output path
+      \param path path to style output file
     */
-    CodeGenerator(const string &colourTheme);
+    void setStyleOutputPath(const string& path);
+
+    /** Set encloding
+      \param encodingName encoding name
+    */
+    void setEncoding(const string& encodingName);
+
+    /** \return style input file path */
+    const string&  getStyleInputPath();
+
+    /** \return style output file path */
+    const string&  getStyleOutputPath();
+
+protected:
+    CodeGenerator(highlight::OutputType type);
+    CodeGenerator() {};
 
     /** \param c Character to be masked
         \return Escape sequence of output format */
@@ -206,12 +259,6 @@ protected:
         \return Found state (integer value)  */
     State getState(const string &s, unsigned int searchPos);
 
-    /** \return Next identifier in current line of code */
-    string getIdentifier();
-
-    /** \return Next number in current line of code */
-    string getNumber();
-
     /** Insert line number at the beginning of current output line */
     virtual void insertLineNumber(bool insertNewLine=true);
 
@@ -230,7 +277,6 @@ protected:
       \return line number  */
     unsigned int getLineNumber();
 
-
     /** Tag Delimiters for every colour style*/
     vector <string> styleTagOpen, styleTagClose;
 
@@ -255,6 +301,12 @@ protected:
     /** Tags which enclose white space indentation blocks */
     string maskWsBegin, maskWsEnd;
 
+    /** Style comment delimiters */
+    string styleCommentOpen, styleCommentClose;
+
+    /** Encoding name */
+    string encoding;
+
     /** Test if maskWsBegin and maskWsEnd should be applied */
     bool maskWs;
 
@@ -270,16 +322,22 @@ protected:
     /** Test if leading spyce of line number should be filled with zeroes*/
     bool lineNumberFillZeroes;
 
+    // -- Kafka begins
+    /** The base font to use */
+    string baseFont ;
+
+    /** The base font size to use */
+    string baseFontSize ;
+// -- Kafka ends
+
     /** Current line of input file*/
     string line;
 
     /** Current line number */
     unsigned int lineNumber;
 
-    // Zeigt den aktuellen Zustand an
-    // wird nicht in getCurrentState gesetzt, da nur Zustände interessant
-    // sind, die als Index auf die styleCloseTags und styleOpenTags verwendet
-    // werden knnen
+    unsigned int lineNumberOffset;
+
     /** Current state*/
     State currentState;
 
@@ -289,11 +347,12 @@ protected:
     /** Processes origin state */
     void processRootState();
 
-    /** return line break sequence */
+    /** \return line break sequence */
     virtual string getNewLine();
 
     /**
        \param s current state
+       \param kwClassID keyword class (has to be set when s=KEYWORD)
        \return Index of style tag corresponding to the states
     */
     unsigned int getStyleID(State s, unsigned int kwClassID = 0);
@@ -301,9 +360,28 @@ protected:
     /** \return line index */
     unsigned int  getLineIndex();
 
+    /** print all remaining white space*/
+    void flushWs();
+
+    /** \return Content of user defined input style */
+    string readUserStyleDef();
+
+    /** \return Style definition of the chosen output format */
+    virtual string  getStyleDefinition() {return "";};
+
+    /** \return true id encoding is defined */
+    bool encodingDefined() {return StringTools::lowerCase(encoding)!="none";}
+
+    /** contains white space, which will be printed after a closing tag */
+    string wsBuffer;
+
+    /** Flag to test if style definition should be included in output document */
+    bool includeStyleDef;
+
 private:
 
-    CodeGenerator(const CodeGenerator&){}
+    CodeGenerator(const CodeGenerator&){cerr<< "Doh!\n";}
+
     CodeGenerator& operator=(CodeGenerator&){ return *this;}
 
     static CodeGenerator* generator;
@@ -312,26 +390,29 @@ private:
     virtual string getMatchingOpenTag(unsigned int) = 0;
     virtual string getMatchingCloseTag(unsigned int) = 0;
 
+    /** Set output type
+      \param s output type
+    */
+    //void setType(OutputType t);
+
+
    /** open a new tag, set current state to s*/
     void openTag(State s);
 
     /** close opened tag, clear current state */
     void closeTag(State s);
 
-
     void closeTag(unsigned int styleID);
 
     void openTag(unsigned int styleID);
 
-    /** print all remaining white space*/
-    void flushWs();
-
-    // path to sytle definition file
+    // path to style definition file
     string  stylePath;
 
     // contains current position in line
     unsigned int lineIndex;
 
+    unsigned int lineNumberWidth;
 
     /**last character of the last line*/
     unsigned char terminatingChar;
@@ -345,14 +426,14 @@ private:
     /** Flag to test if formatting is enabled with current input document*/
     bool formattingEnabled;
 
+
     /** Flag to test if formatting is possible with current input document*/
     bool formattingPossible;
 
     /** contains the current token*/
     string token;
 
-    /** contains white space, which will be printed after a closing tag */
-    string wsBuffer;
+    string styleInputPath, styleOutputPath;
 
     /** Resets parser to origin state, call this after every file conversion */
     void reset();
@@ -363,6 +444,8 @@ private:
     /** return next character from in stream */
     unsigned char getInputChar();
 
+    OutputType outputType;
+
    /** return new state */
     State getCurrentState ( bool lastStateWasNumber=false);
 
@@ -378,23 +461,40 @@ private:
     bool processSymbolState();
     void processWsState();
 
-    /** gibt true zurck, falls c ein erlaubter Character innerhalb von Keyword
-        oder Typbezeichner ist */
-    bool isAllowedChar(char c) ;
-
     /** returns true if curret token is the first in line and no whitespace */
     bool isFirstNonWsChar() ;
 
     /** print escaped token and clears it */
     void printMaskedToken(bool flushWhiteSpace=true);
 
-    /** print escape sequence */
-    void skipEscapeSequence();
-
     void closeKWTag(unsigned int styleID);
+
     void openKWTag(unsigned int styleID);
 
+    /** look for special commands in comments, take action in derived class
+        \return true if command was found
+    */
+    bool checkSpecialCmd();
+
+    map <int, ReGroup*> regexGroups;
+
+    void matchRegex(const string &);
+
   };
+
+  class ReGroup {
+  public:
+
+    ReGroup(State s, unsigned int l , unsigned int c ): length(l), state(s), kwClass(c) {
+      //cerr << "RE len "<<l<< "  state "<<s<<"  class "<<c <<endl;
+    }
+    ~ReGroup(){}
+
+    unsigned int length;
+    State state;
+    unsigned int kwClass;
+  };
+
 }
 
 #endif
