@@ -23,47 +23,36 @@ ConfigurationReader::ConfigurationReader(const string & configuration_path)
 {
   ifstream in (configuration_path.c_str());
   fileFound=in;
-  if (fileFound)
-    {
-      string line;
-      line.reserve(500);
-      unsigned int  lineBegin;
-      size_t delimPos;
-      string paramName, paramValue;
-      while (getline(in, line))
-        {
-          lineBegin=line.find_first_not_of("\t ");
-          if ((line.size()>2) && (lineBegin!=string::npos)
-               && (line.at(lineBegin)!='#'))  //comment?
-            {
-              if (line[lineBegin]=='$')
-                {             // neuer Parametername?
-                  delimPos=line.find("=",lineBegin)-1;
-                  if (delimPos!=string::npos)
-                    {
-                      paramName=StringTools::lowerCase(line.substr(lineBegin+1,
-                                                       delimPos));
-                      parameterNames.push_back(paramName);
-                      paramValue=line.substr(delimPos+2, line.length());
-                    }
-                }
-              else // line belongs to last parameter
-                {
-                  paramValue=line;
-                }
-
-              if (parameterMap[paramName].empty())
-                {
-                  parameterMap[paramName] = paramValue;
-                }
-              else
-                {
-                  parameterMap[paramName]+= (" "+paramValue);
-                }
-            }  //if ((lineBegin!=string::npos) && (line.at(lineBegin)!='#'))
-        } //while
-      in.close();
-    } //if (in)
+  if (fileFound) {
+    string line;
+    line.reserve(500);
+    unsigned int  lineBegin;
+    size_t delimPos;
+    string paramName, paramValue;
+    while (getline(in, line)) {
+      lineBegin=line.find_first_not_of("\t ");
+      if ((line.size()>2) && (lineBegin!=string::npos)
+            && (line.at(lineBegin)!='#')) {  //comment?
+          if (line[lineBegin]=='$') {       // neuer Parametername?
+             delimPos=line.find("=",lineBegin)-1;
+             if (delimPos!=string::npos) {
+                 paramName=StringTools::trimRight(
+                           StringTools::lowerCase(line.substr(lineBegin+1, delimPos)));
+                 parameterNames.push_back(paramName);
+                 paramValue=line.substr(delimPos+2, line.length());
+              }
+            } else  { // line belongs to last parameter
+                paramValue=line;
+            }
+            if (parameterMap[paramName].empty())  {
+               parameterMap[paramName] = paramValue;
+            } else {
+                parameterMap[paramName]+= (" "+paramValue);
+            }
+         }  //if ((lineBegin!=string::npos) && (line.at(lineBegin)!='#'))
+      } //while
+    in.close();
+  } //if (in)
 }
 
 ConfigurationReader::~ConfigurationReader()
