@@ -2,7 +2,7 @@
                           main.cpp  -  description
                              -------------------
     begin                : Die Apr 23 22:16:35 CEST 2002
-    copyright            : (C) 2002-2004 by Andr Simon
+    copyright            : (C) 2002-2004 by Andre Simon
     email                : andre.simon1@gmx.de
 
 
@@ -22,6 +22,9 @@
  ***************************************************************************/
 
 #include "main.h"
+#include "re/Pattern.h"
+
+#define MAX_LINE__WIDTH       80
 
 using namespace std;
 
@@ -29,18 +32,23 @@ void HighlightApp::printVersionInfo()
 {
   cout << "\n highlight version "
        << HIGHLIGHT_VERSION
-       #ifdef USE_LOCAL_GETOPT
-         << "\n\n (Built with USE_LOCAL_GETOPT flag.)\n"
-       #endif
-       << "\n Copyright (C) 2002-2005 Andre Simon <andre.simon1@gmx.de>"
+       << "\n Copyright (C) 2002-2005 Andre Simon <andre.simon1 at gmx.de>"
        << "\n\n Artistic Style Classes (1.15.3)"
-       << "\n Copyright (C) 1998-2002 Tal Davidson <davidsont@bigfoot.com>"
+       << "\n Copyright (C) 1998-2002 Tal Davidson <davidsont at bigfoot.com>"
        << "\n\n Dirstream Classes (0.4)"
-       << "\n Copyright (C) 2002-2004 Benjamin Kaufmann <hume@c-plusplus.de>"
+       << "\n Copyright (C) 2002-2004 Benjamin Kaufmann <hume at c-plusplus.de>"
+       << "\n\n Regex library (0.02a)"
+       << "\n Copyright (C) 2003-2005 Jeffery Stuart <stuart at cs.unr.edu>"
        << "\n\n This software is released under the terms of the GNU General "
        << "Public License."
        << "\n For more information about these matters, see the file named "
        << "COPYING.\n\n";
+       #ifdef USE_LOCAL_GETOPT
+         cout   << " (Built with USE_LOCAL_GETOPT flag set.)\n";
+       #endif
+       #ifdef HL_DATA_DIR
+        cout    << " (HL_DATA_DIR: \"" <<HL_DATA_DIR<< "\" )\n";
+       #endif
 }
 
 void HighlightApp::printBadInstallationInfo()
@@ -49,7 +57,7 @@ void HighlightApp::printBadInstallationInfo()
        << OPT_DATADIR << " parameter."
        << "\n\nCopy the highlight files into one of the directories listed "
        << "in INSTALL.\nYou may also set the data directory with "
-       << OPT_DATADIR << "and " << OPT_ADDDATADIR << ".\n";
+       << OPT_DATADIR << " and " << OPT_ADDDATADIR << ".\n";
 }
 
 bool HighlightApp::listInstalledFiles(bool showThemes)
@@ -98,8 +106,13 @@ void HighlightApp::printDebugInfo(highlight::LanguageDefinition &lang,
                                    const string & langDefPath)
 {
    cerr << "\nLoading language definition: " << langDefPath;
-   cerr << "\n\nSYMBOLS: "
-        << lang.getSymbolString();
+   cerr << "\n\nSYMBOLS:\n" << lang.getSymbolString();
+   cerr << "\n\nREGEX:\n";
+   highlight::RegexElement *re=NULL;
+   for (unsigned int i=0; i<lang.getRegexElements().size(); i++){
+      re = lang.getRegexElements()[i];
+      cerr << "State "<<re->open<<":\t"<<re->rePattern->getPattern()<<"\n";
+   }
    cerr << "\n\nKEYWORDS: ";
    highlight::KeywordMap::iterator it;
    highlight::KeywordMap keys=lang.getKeywords();
@@ -118,35 +131,28 @@ string HighlightApp::getFileSuffix(const string &fileName) {
          "" : fileName.substr(ptPos+1, fileName.length());
 }
 
-
-bool HighlightApp::loadFileExtensions(){
-  string extPath=dataDir.getDir() + "extensions.conf";
-  ConfigurationReader extensionsConfig(extPath);
-  if (extensionsConfig.found())
-  {
-    stringstream values;
-    string paramName, paramVal;
-    for (unsigned int i=0;i<extensionsConfig.getParameterNames().size();i++){
-      paramName = extensionsConfig.getParameterNames()[i];
-      values.str(extensionsConfig.getParameter(paramName)) ;
-      while (values >> paramVal) {
-         extensions[ paramVal ] = paramName;
-      }
-      values.clear();
+bool HighlightApp::loadMapConfig(const string& name, StringMap* map){
+    string extPath=dataDir.getConfDir() + name + ".conf";
+    ConfigurationReader config(extPath);
+    if (config.found() )
+    {
+        stringstream values;
+        string paramName, paramVal;
+        for (unsigned int i=0;i<config.getParameterNames().size();i++){
+            paramName = config.getParameterNames()[i];
+            values.str(config.getParameter(paramName)) ;
+            while (values >> paramVal) {
+                map->insert(make_pair( paramVal,  paramName));
+            }
+            values.clear();
+        }
+        return true;
+    } else {
+        cerr << "highlight: Configuration file "<< extPath << " not found.\n";
+        return false;
     }
-    return true;
-  } else {
-    cerr << "highlight: Extension configuration file "
-         << extPath
-         << " not found.\n";
-    return false;
-  }
 }
 
-string HighlightApp::getFileType(const string& suffix)
-{
-  return (extensions.count(suffix)) ? extensions[suffix] : suffix ;
-}
 
 int HighlightApp::getNumDigits(int i){
   int res=0;
@@ -201,9 +207,32 @@ void HighlightApp::printIOErrorReport(unsigned int numberErrorFiles,
   }
 }
 
+string HighlightApp::analyzeFile(const string& file){
+    if (scriptShebangs.empty()) loadMapConfig("scriptre", &scriptShebangs);
+    ifstream inFile(file.c_str());
+    string firstLine;
+    getline (inFile, firstLine);
+    StringMap::iterator it;
+     for (it=scriptShebangs.begin(); it!=scriptShebangs.end();it++){
+       if (Pattern::matches(it->first, firstLine)) return it->second;
+     }
+    return "";
+}
+
+string HighlightApp::guessFileType(const string& suffix, const string &inputFile)
+{
+    if (extensions.empty()) loadMapConfig("extensions", &extensions);
+    string fileType = (extensions.count(suffix)) ? extensions[suffix] : suffix ;
+    if (!fileType.empty()) return fileType;
+    return analyzeFile(inputFile);
+}
+
 
 int HighlightApp::run(int argc, char**argv){
 
+    //cerr << "***THIS IS A BETA RELEASE***\nPlease use the contact form at www.andre-simon.de to report bugs,\n";
+    //cerr <<"especially concerning the new regex features.\n";
+
   //get command line options
   CmdLineOptions options(argc, argv);
 
@@ -221,8 +250,6 @@ int HighlightApp::run(int argc, char**argv){
      dataDir.setAdditionalDataDir(options.getAdditionalDataDir());
   }
 
-  loadFileExtensions();
-
   if (options.printVersion()) {
     printVersionInfo();
     return EXIT_SUCCESS;
@@ -243,23 +270,34 @@ int HighlightApp::run(int argc, char**argv){
   string stylePath=dataDir.searchForTheme(options.getStyleName());
 
   highlight::CodeGenerator *generator =
-    highlight::CodeGenerator::getInstance(options.getOutputType(),
-                                          stylePath,
-                                          options.getCssInFilename(),
-                                          options.getCssOutFilename(),
-                                          options.includeCssDefinition(),
-                                          options.attachLineAnchors(),
-                                          options.replaceQuotes(),
-                                          options.fopCompatible(),
-                                          options.getNumberSpaces(),
-                                          options.getWrappingStyle(),
-                                          options.printLineNumbers(),
-                                          options.fillLineNrZeroes(),
-                                          options.fragmentOutput());
+    highlight::CodeGenerator::getInstance(options.getOutputType());
+
+  generator->initStyle(stylePath);
+  generator->setStyleInputPath(options.getStyleInFilename());
+  generator->setStyleOutputPath(options.getStyleOutFilename());
+  generator->setIncludeStyle(options.includeStyleDef());
+  generator->setPrintLineNumbers(options.printLineNumbers(), options.getNumberStart());
+  generator->setPrintZeroes(options.fillLineNrZeroes());
+  generator->setFragmentCode(options.fragmentOutput());
+  generator->setPreformatting(options.getWrappingStyle(),
+                             (generator->getPrintLineNumbers()) ?
+                              MAX_LINE__WIDTH - options.getNumberWidth() : MAX_LINE__WIDTH,
+                              options.getNumberSpaces() );
+
+  generator->setEncoding(options.getEncoding());
+  generator->setBaseFont(options.getBaseFont()) ;
+  generator->setBaseFontSize(options.getBaseFontSize()) ;
+  generator->setLineNumberWidth(options.getNumberWidth());
+
+  generator->setSpecialOptions(options.attachLineAnchors(),options.orderedList(),
+                               options.replaceQuotes(), options.disableBabelShorthands(),
+                               options.fopCompatible());
 
   assert (generator!=NULL);
 
-  if (!generator->styleFound()) {
+  bool styleFileWanted = !options.fragmentOutput() || options.styleOutPathDefined();
+
+  if (!generator->styleFound() ) {
     cerr << "highlight: Could not find style "
          << stylePath
          << ".\n";
@@ -287,32 +325,54 @@ int HighlightApp::run(int argc, char**argv){
      return EXIT_FAILURE;
   }
 
+  bool initError=false, IOError=false;
+
+  if (    !options.includeStyleDef()
+       && styleFileWanted
+       && options.formatSupportsExtStyle()) {
+      string cssOutFile=outDirectory  + options.getStyleOutFilename();
+      bool success=generator->printExternalStyle (cssOutFile);
+      if (!success){
+          cerr << "highlight: Could not write " << cssOutFile <<".\n";
+          IOError = true;
+      }
+  }
+
+  if (options.printIndexFile()){
+    bool success=generator -> printIndexFile(inFileList, outDirectory);
+    if (!success){
+      cerr << "highlight: Could not write index file.\n";
+      IOError = true;
+    }
+  }
+
   unsigned int fileCount=inFileList.size(),
                fileCountWidth=getNumDigits(fileCount),
                i=0,
                numBadFormatting=0,
                numBadInput=0,
                numBadOutput=0;
-  bool initError=false, IOError=false;
+
   vector<string> badFormattedFiles, badInputFiles, badOutputFiles;
   string outFilePath;
   string suffix, lastSuffix;
 
-  if (options.syntaxGiven()) {
-    suffix = getFileType(options.getLanguage());
+  if (options.syntaxGiven()) {  // user defined language definition, valid for all files
+      suffix = guessFileType(options.getLanguage());
   }
 
   while (i < fileCount && !initError) {
-    if (!options.syntaxGiven()) {
-      suffix = getFileType(getFileSuffix(inFileList[i]));
+    if (!options.syntaxGiven()) {  // determine file type for each file
+       suffix = guessFileType(getFileSuffix(inFileList[i]), inFileList[i]);
     }
-
     if (suffix.empty()) {
-            if (!options.enableBatchMode())
-                cerr << "highlight: Undefined language definition. Use --"
-                    << OPT_SYNTAX << " option.\n";
-            initError = true;
-            break;
+      if (!options.enableBatchMode())
+        cerr << "highlight: Undefined language definition. Use --"
+             << OPT_SYNTAX << " option.\n";
+        if (!options.forceOutput()){
+          initError = true;
+          break;
+        }
     }
 
     if (suffix != lastSuffix) {
@@ -322,8 +382,10 @@ int HighlightApp::run(int argc, char**argv){
             cerr << "highlight: Unknown source file extension \""
                 << suffix
                 << "\".\n";
-            initError = true;
-            break;
+            if (!options.forceOutput()){
+              initError = true;
+              break;
+            }
         }
         if (options.printDebugInfo() && loadRes==highlight::LOAD_NEW){
             printDebugInfo(generator->getLanguage(), langDefPath);
@@ -348,7 +410,8 @@ int HighlightApp::run(int argc, char**argv){
         outFilePath = options.getSingleOutFilename();
      }
 
-     highlight::ParseError error = generator->printOutput(inFileList[i], outFilePath);
+     highlight::ParseError error = generator->generateFile(inFileList[i], outFilePath);
+
      if (error==highlight::BAD_INPUT){
        if (numBadInput++ < IO_ERROR_REPORT_LENGTH || options.printDebugInfo()) {
          badInputFiles.push_back(inFileList[i]);
@@ -366,23 +429,6 @@ int HighlightApp::run(int argc, char**argv){
      ++i;
     }
 
-    if (!options.includeCssDefinition() && !options.fragmentOutput()) {
-      string cssOutFile=outDirectory + options.getCssOutFilename();
-      bool success=generator->printExternalStyle (cssOutFile);
-      if (!success){
-        cerr << "highlight: Could not write " << cssOutFile <<".\n";
-        IOError = true;
-      }
-    }
-
-    if (options.printIndexFile()){
-        bool success=generator -> printIndexFile(inFileList, outDirectory);
-        if (!success){
-          cerr << "highlight: Could not write index file.\n";
-          IOError = true;
-        }
-    }
-
     if (numBadInput){
       printIOErrorReport(numBadInput, badInputFiles, "read input");
       IOError = true;
@@ -399,6 +445,7 @@ int HighlightApp::run(int argc, char**argv){
     return (initError || IOError) ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
+
 int main(int argc, char **argv) {
   HighlightApp app;
   return app.run(argc, argv);
