@@ -13,6 +13,7 @@
 #ifndef HIGHLIGHT_APP
 #define HIGHLIGHT_APP
 
+
 #include <iostream>
 #include <fstream>
 #include <string>
@@ -22,7 +23,6 @@
 #include <cassert>
 
 #include "./dirstream0.4/dirstream.h"
-
 #include "cmdlineoptions.h"
 #include "configurationreader.h"
 #include "codegenerator.h"
@@ -32,8 +32,9 @@
 #include "platform_fs.h"
 
 #define IO_ERROR_REPORT_LENGTH 5
+#define SHEBANG_CNT 12
 
-typedef map<string, string> ExtensionsMap;
+typedef map<string, string> StringMap;
 
 /** Main application class
    @author Andre Simon
@@ -42,6 +43,7 @@ typedef map<string, string> ExtensionsMap;
 class HighlightApp {
 
 public:
+
    HighlightApp(){};
    ~HighlightApp(){};
 
@@ -55,7 +57,8 @@ public:
 private:
 
   DataDir dataDir;
-  ExtensionsMap extensions;
+  StringMap extensions;
+  StringMap scriptShebangs;
 
   /** print version info*/
   void printVersionInfo();
@@ -74,11 +77,9 @@ private:
   void printDebugInfo(highlight::LanguageDefinition &lang,
                       const string &langDefPath);
 
-  string getFileSuffix(const string &fileName) ;
+  string getFileSuffix(const string &fileName);
 
-  bool loadFileExtensions();
-
-  string getFileType(const string &suffix);
+  string guessFileType(const string &suffix, const string &inputFile="");
 
   int getNumDigits(int i);
 
@@ -89,6 +90,9 @@ private:
   bool readInputFilePaths(vector<string> &fileList, string wildcard,
                           bool recursiveSearch);
 
+  string analyzeFile(const string& file);
+  bool loadMapConfig(const string& name, StringMap* map);
+
 };
 
 #endif
