@@ -19,8 +19,8 @@
 
 namespace highlight {
 
-TexGenerator::TexGenerator(const string &colourTheme):
-  CodeGenerator( colourTheme)
+TexGenerator::TexGenerator()
+: CodeGenerator(TEX)
 {
   styleTagOpen.push_back( "{\\hlstd ");
   styleTagOpen.push_back( "{\\hlstr ");
@@ -46,10 +46,9 @@ TexGenerator::TexGenerator(const string &colourTheme):
   excludeWs=true;
   maskWsBegin = "{\\hlstd";
   maskWsEnd = "}";
+  styleCommentOpen="%";
 }
 
-TexGenerator::TexGenerator()
-{}
 TexGenerator::~TexGenerator()
 {}
 
@@ -73,36 +72,25 @@ string TexGenerator::formatStyleAttributes(const string & elemName,const Element
 string TexGenerator::getHeader(const string & title)
 {
   ostringstream os;
-  os << formatStyleAttributes("std", docStyle.getDefaultStyle());
-  os << formatStyleAttributes("num", docStyle.getNumberStyle());
-  os << formatStyleAttributes("esc", docStyle.getEscapeCharStyle());
-  os << formatStyleAttributes("str", docStyle.getStringStyle());
-  os << formatStyleAttributes("dstr", docStyle.getDirectiveStringStyle());
-  os << formatStyleAttributes("slc", docStyle.getSingleLineCommentStyle());
-  os << formatStyleAttributes("com", docStyle.getCommentStyle());
-  os << formatStyleAttributes("dir", docStyle.getDirectiveStyle());
-  os << formatStyleAttributes("line", docStyle.getLineStyle());
-  os << formatStyleAttributes("sym", docStyle.getSymbolStyle());
 
-  KeywordStyles styles = docStyle.getKeywordStyles();
-  for (KSIterator it=styles.begin(); it!=styles.end(); it++){
-    os << formatStyleAttributes(it->first, *(it->second));
+  if (langInfo.getSyntaxHighlight()) {
+    if (includeStyleDef) {
+       os << "\n"<<getStyleDefinition();
+       os << CodeGenerator::readUserStyleDef();
+    }  else  {
+       os << "\\input "
+          << getStyleOutputPath()
+          << "\n\n";
+    }
   }
-  os << "\\nopagenumbers\n"
-     << "\\input colordvi\n";
 
   return os.str();
 }
 
 void TexGenerator::printBody()
 {
-  *out << "{\n\\tt\n"
-       <<  "% The special option is not supported by all dvi drivers\n"
-       << "\\special{background rgb ";
-  *out << docStyle.getBgColour().getLatexRedValue() << " "
-       << docStyle.getBgColour().getLatexGreenValue() << " "
-       << docStyle.getBgColour().getLatexBlueValue() << "}"
-       << endl;
+    *out << "{\n\\"<<getBaseFont()<<"\n";
+
   processRootState();
   *out << "}\n";
 }
@@ -153,9 +141,9 @@ string TexGenerator:: maskCharacter(unsigned char c)
     case '\\':
       return "$\\backslash$";
       break;
-   /* case ' ':
+    case ' ':
       return spacer;
-      break;*/
+      break;
     case '+':
     case '-':
     case '<':
@@ -238,12 +226,6 @@ string TexGenerator:: maskCharacter(unsigned char c)
     case SZLIG:
       return "\\ss ";
       break;
-     #ifndef _WIN32
- // skip  first byte of multibyte chracters
-    case 195:
-      return string("");
-      break;
-     #endif
 
     default :
       string m;
@@ -260,4 +242,37 @@ string TexGenerator::getMatchingCloseTag(unsigned int styleID){
 }
 
 
+string TexGenerator::getStyleDefinition()
+{
+    if (styleDefinitionCache.empty()){
+        ostringstream os;
+        os << formatStyleAttributes("std", docStyle.getDefaultStyle());
+        os << formatStyleAttributes("num", docStyle.getNumberStyle());
+        os << formatStyleAttributes("esc", docStyle.getEscapeCharStyle());
+        os << formatStyleAttributes("str", docStyle.getStringStyle());
+        os << formatStyleAttributes("dstr", docStyle.getDirectiveStringStyle());
+        os << formatStyleAttributes("slc", docStyle.getSingleLineCommentStyle());
+        os << formatStyleAttributes("com", docStyle.getCommentStyle());
+        os << formatStyleAttributes("dir", docStyle.getDirectiveStyle());
+        os << formatStyleAttributes("line", docStyle.getLineStyle());
+        os << formatStyleAttributes("sym", docStyle.getSymbolStyle());
+
+        KeywordStyles styles = docStyle.getKeywordStyles();
+        for (KSIterator it=styles.begin(); it!=styles.end(); it++){
+            os << formatStyleAttributes(it->first, *(it->second));
+        }
+
+        os <<  "% The special option is not supported by all dvi drivers\n"
+           << "\\special{background rgb "
+           << docStyle.getBgColour().getLatexRedValue() << " "
+           << docStyle.getBgColour().getLatexGreenValue() << " "
+           << docStyle.getBgColour().getLatexBlueValue() << "}";
+        os << "\n\\nopagenumbers\n"
+           << "\\input colordvi\n";
+        styleDefinitionCache=os.str();
+    }
+    return styleDefinitionCache;
+}
+
+
 }
