@@ -1,3 +1,4 @@
+
 /***************************************************************************
                           xslfocode.cpp  -  description
                              -------------------
@@ -20,10 +21,8 @@
 using namespace std;
 namespace highlight {
 
-XslFoGenerator::XslFoGenerator(const string &colourTheme,
-                               bool fopCompatible)
-    : CodeGenerator(colourTheme),
-     fopOutput(fopCompatible)
+XslFoGenerator::XslFoGenerator()
+: CodeGenerator(XSLFO)
 {
   styleTagOpen.push_back( getOpenTag(docStyle.getDefaultStyle()));
   styleTagOpen.push_back( getOpenTag(docStyle.getStringStyle()));
@@ -35,9 +34,12 @@ XslFoGenerator::XslFoGenerator(const string &colourTheme,
   styleTagOpen.push_back( getOpenTag(docStyle.getDirectiveStringStyle()));
   styleTagOpen.push_back( getOpenTag(docStyle.getLineStyle()));
   styleTagOpen.push_back( getOpenTag(docStyle.getSymbolStyle()));
-  snl << "  <fo:block font-size=\""
-      << docStyle.getFontSize()
-      << "pt\" font-family=\"Courier\" white-space-collapse=\"false\" "
+  snl << "  <fo:block font-size=\"";
+  snl << this->getBaseFontSize();
+  snl << "pt\" font-family=\"";
+  snl << this->getBaseFont();
+
+  snl << "\" white-space-collapse=\"false\" "
       << "wrap-option=\"wrap\" line-height=\"12pt\" background-color=\"#"
       << (docStyle.getBgColour().getHexRedValue())
       << (docStyle.getBgColour().getHexGreenValue())
@@ -56,8 +58,6 @@ XslFoGenerator::XslFoGenerator(const string &colourTheme,
   spacer = " ";
 }
 
-XslFoGenerator::XslFoGenerator()
-{}
 XslFoGenerator::~XslFoGenerator()
 {}
 
@@ -79,8 +79,11 @@ string  XslFoGenerator::getOpenTag(const ElementStyle &elem)
 string XslFoGenerator::getHeader(const string & title)
 {
    ostringstream os;
-   os << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"
-      << "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n"
+   os << "<?xml version=\"1.0\"";
+   if (encodingDefined()) {
+       os << " encoding=\"" << encoding << "\"";
+   }
+   os << "?>\n<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n"
       << "<fo:layout-master-set>\n"
       << "<fo:simple-page-master master-name=\"DINA4\"\n"
       << "    page-height=\"29.7cm\"\n"
@@ -145,14 +148,6 @@ string XslFoGenerator::maskCharacter(unsigned  char c)
     case '\"' :
       return "&quot;";
       break;
-
-// skip  first byte of multibyte chracters
-    #ifndef _WIN32
-    case 195:
-      return string("");
-      break;
-    #endif
-
     default:
       string m;
       m += c;
@@ -160,18 +155,18 @@ string XslFoGenerator::maskCharacter(unsigned  char c)
     }
 }
 
-string XslFoGenerator::getNewLine(){
+/*string XslFoGenerator::getNewLine(){
   string nlStr;
 
   if (currentState!=_UNKNOWN){
       nlStr+=styleTagClose[getStyleID(currentState, currentKeywordClass)];
-  }
+}
   nlStr += newLineTag;
   if (currentState!=_UNKNOWN){
       nlStr+=styleTagOpen[getStyleID(currentState, currentKeywordClass)];
-  }
-  return nlStr;
 }
+  return nlStr;
+}*/
 
 string XslFoGenerator::getMatchingOpenTag(unsigned int styleID){
  return getOpenTag(docStyle.getKeywordStyle(langInfo.getKeywordClasses()[styleID]));
